// $Id: Abenteurer.cc,v 1.32 2005/04/27 08:54:45 thoma Exp $
/*  Midgard Character Generator
 *  Copyright (C) 2002 Malte Thoma
 *  Copyright (C) 2003-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Abenteurer.hh"
//#include "export_common.h"
#include <Misc/TagStream.h>
#include "Preise.hh"
#include <Misc/itos.h>
#include <Sprache.hh>
#include <Schrift.hh>
#include "Fertigkeiten_angeboren.hh"
#include "Waffe.hh"
#include "WaffeGrund.hh"
#include "Zauber.hh"
#include "Zauberwerk.hh"
#include "KiDo.hh"
#include "Beruf.hh"
#include <Misc/Trace.h>
#include "magustrace.h"
#include "Datenbank.hh"
#include "Ausgabe.hh"
#include <libmagus/LernListen.hh>

#include <Magus_Optionen.hh>
Abenteurer::Abenteurer(bool initialize) : Grundwerte(initialize)
{ if (initialize)
  { for(Magus_Optionen::regionen_t::const_iterator i=Programmoptionen->standard_regionen.begin();i!=Programmoptionen->standard_regionen.end();++i)
      regionen[i->first] = i->second;
    for(std::list<Optionen::st_Haus>::iterator i=getOptionen().getHausregeln().begin();
                  i!=getOptionen().getHausregeln().end();++i)
      i->active=Programmoptionen->standard_erweiterungen[i->text];
    for(std::list<Optionen::st_OptionenCheck>::iterator i=getOptionen().getOptionenCheck().begin();
                  i!=getOptionen().getOptionenCheck().end();++i)
      i->active=Programmoptionen->standard_erweiterungen[i->text];
  }
}

void Abenteurer::Originalregeln()
{ for(Magus_Optionen::regionen_t::const_iterator i=Programmoptionen->standard_regionen.begin();i!=Programmoptionen->standard_regionen.end();++i)
      regionen[i->first] = false;
  for(std::list<Optionen::st_Haus>::iterator i=getOptionen().getHausregeln().begin();
                  i!=getOptionen().getHausregeln().end();++i)
      i->active=false;
  for(std::list<Optionen::st_OptionenCheck>::iterator i=getOptionen().getOptionenCheck().begin();
                  i!=getOptionen().getOptionenCheck().end();++i)
      i->active=i->index==Optionen::Original;
}

bool Abenteurer::Valid() const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  if(getVTyp().size()!=2) return false;
  return Typ1()->Valid();
}


const std::string Abenteurer::LastSavedAt() const
{
  tm *t=localtime(&last_saved_time);
  return itos(t->tm_mday) +"."+ itos(t->tm_mon+1) +"."+ itos(1900+t->tm_year);
}

std::string Grundwerte::STyp() const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  if(Typ2()->Name(Geschlecht())!="")
   return Typ1()->Name(Geschlecht())+"/"+Typ2()->Name(Geschlecht());
  else return Typ1()->Name(Geschlecht());
  abort(); //never get here
}


const std::string Abenteurer::SErfolgswert(std::string name) const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  std::pair<int,bool> w=Erfolgswert(name);
  if(w.first==-99) return "";
  if(w.second) return itos(w.first);
  else return "("+itos(w.first)+")";
}

const std::pair<int,bool> Abenteurer::Erfolgswert(std::string name) const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  for(std::list<H_MBEm>::const_iterator i=list_Fertigkeit.begin();i!=list_Fertigkeit.end();++i)
   {
     if(name==(*(*i))->Name()) return std::pair<int,bool>((*i)->Erfolgswert(),true);
   }
  std::list<st_universell> UF=List_Universell();
  for(std::list<st_universell>::const_iterator i=UF.begin();i!=UF.end();++i)
   {
     if(name==(*i->mbe)->Name()) return std::pair<int,bool>(i->mbe->Erfolgswert(),false);
   }
  return std::pair<int,bool>(-99,false);
}


const std::list<Abenteurer::st_universell> Abenteurer::List_Universell() const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  std::list<st_universell> UF;
  for(std::list<cH_MBE>::const_iterator i=Datenbank.Fertigkeit.begin();i!=Datenbank.Fertigkeit.end();++i)
   {
     cH_Fertigkeit f(*i);
     if(f->Ungelernt(*this)!=-99)
     UF.push_back(H_MBEm(*i));
   }
  cH_MBE werfen(cH_Waffe("Werfen"));
  UF.push_back(H_MBEm(werfen));
  UF.push_back(Ueberleben());
  UF.sort(sort_universell());

  for(std::list<Abenteurer::st_universell>::iterator i=UF.begin();i!=UF.end();++i)
   {
     int iwert=-99;
     if ((*i->mbe)->What()==MBE_T::FERTIGKEIT)
      {
        cH_Fertigkeit f(i->mbe->getMBE());
        iwert = f->Ungelernt(*this);
        if(f->Name()==(*(Ueberleben()))->Name()) iwert=6;
        if (!f->Voraussetzung(*this,true))
            {iwert-=2; i->voraussetzung=false;}
      }
     else if ((*i->mbe)->What()==MBE_T::WAFFE)
      {
        cH_Waffe f((*i->mbe).getMBE());
        iwert = 4+bo_An();
        if (!f->Voraussetzung(*this,true))
            {iwert=0; i->voraussetzung=false;}
      }
     if ((*i->mbe).ist_gelernt(List_Fertigkeit()) || (*i->mbe).ist_gelernt(List_Waffen()))
          i->gelernt=true;
     else // Erfolgswert nur dann setzen, wenn die Fertigkeit NICHT gelernt ist
          (*i->mbe).setErfolgswert(iwert);
   }
  return UF;
}

bool Grundwerte::is_mage() const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
   if(getVTyp().size()!=2) return false;
   if(Typ1()->is_mage() || Typ2()->is_mage()) return true;
   else return false;
}



void Abenteurer::setAngebFert()
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  List_Fertigkeit_ang().clear();
  List_Fertigkeit_ang()=Spezies()->getAngFertigkeiten();
  List_Zauber()=Spezies()->getZauber();
  resetSinne();
  std::list<std::pair<std::string,int> > L=Spezies()->getSinne();
  for(std::list<std::pair<std::string,int> >::const_iterator i=L.begin();i!=L.end();++i)
       setSinn(i->first,i->second);
}

bool Abenteurer::setAngebSinnFert(int wurf,const H_MBEm &MBE)
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  int wert=MBE->Erfolgswert();
  if     ( 1<=wurf && wurf<= 2) setSinnCheck("Sehen",wert);
  else if( 3<=wurf && wurf<= 4) setSinnCheck("Hören",wert);
  else if( 5<=wurf && wurf<= 6) setSinnCheck("Riechen",wert);
  else if( 7<=wurf && wurf<= 8) setSinnCheck("Schmecken",wert);
  else if( 9<=wurf && wurf<=10) setSinnCheck("Tasten",wert);
  else if(11<=wurf && wurf<=20) setSinn("Sehen",wert);
  else if(21<=wurf && wurf<=30) setSinn("Hören",wert);
  else if(31<=wurf && wurf<=40) setSinn("Riechen",wert);
  else if(41<=wurf && wurf<=50) setSinn("Schmecken",wert);
  else if(51<=wurf && wurf<=60) setSinn("Tasten",wert);
  else if(61<=wurf && wurf<=65) setSinn("Sechster Sinn",wert);
  else { List_Fertigkeit_ang().push_back(MBE); return false;}
  return true;
}




void Abenteurer::speicherstream(std::ostream &datei) const
{  speichern(datei);
}

void Abenteurer::speichern(std::ostream &datei) const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
   TagStream ts;
   ts.setEncoding("UTF-8");

   Tag &data=ts.push_back(Tag("MAGUS-data"));

   Tag &Abenteurer=data.push_back(Tag("Midgard-Abenteurer"));
   Abenteurer.setIntAttr("Version",10);

   grundwerte_speichern(Abenteurer);
   Tag &Ausruestung=Abenteurer.push_back(Tag("Ausrüstung"));
   Ausruestung.push_back(Tag("Rüstung", Ruestung()->Name()));
   if (!Ruestung(1)->Name().empty())
     Ausruestung.push_back(Tag("Rüstung2", Ruestung(1)->Name()));
   // Waffen Besitz
   for (std::list<H_WaffeBesitz>::const_iterator i=List_Waffen_besitz().begin();
         i!=List_Waffen_besitz().end();++i)
      {
         Tag &w=Ausruestung.push_back(Tag("Waffe"));
         w.setIntAttr("Erfolgswert", (*i)->Erfolgswert());
         w.setAttr("Bezeichnung", (*i)->AliasName());
         w.setIntAttr_nn("AngriffVerteidigung_Bonus", (*i)->av_Bonus());
         w.setIntAttr_nn("SchadenLebenspunkte_Bonus", (*i)->sl_Bonus());
         w.setAttr_ne("Region", (*i)->Region());
         if (!(*i)->Magisch().empty()) w.Value((*i)->Magisch());
      }
   save_ausruestung(Ausruestung, getBesitz().getChildren());

   Tag &Fertigkeiten=Abenteurer.push_back(Tag("Fertigkeiten"));

   MBE_T::saveElementliste(Fertigkeiten,Sinne(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Beruf(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Fertigkeit_ang(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Fertigkeit(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Waffen(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Zauber(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Zauberwerk(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Kido(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_WaffenGrund(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Sprache(),*this,getVTyp());
   MBE_T::saveElementliste(Fertigkeiten,List_Schrift(),*this,getVTyp());
   // Zauber, die von Spruchrolle gelernt wurden
   for(std::list<std::string>::const_iterator i= list_Gelernt_von_Spruchrolle.begin();i!=list_Gelernt_von_Spruchrolle.end();++i)
      Abenteurer.push_back(Tag("Von Spruchrolle gelernt",*i));

   // Regionen & Optionen
   Tag &Opt=Abenteurer.push_back(Tag("Optionen"));
  for(std::map<cH_Region,Model_copyable<bool> >::const_iterator i=regionen.begin();i!=regionen.end();++i)
   {  if (!i->second.Value()) continue;
      Tag &r=Opt.push_back(Tag("Region"));
      r.setAttr("Name", i->first->Name());
      r.setAttr("Region", i->first->Abkuerzung());
   }
   // Optionen
   const std::list<Optionen::st_OptionenCheck> &LO=optionen.getOptionenCheck();
   for(std::list<Optionen::st_OptionenCheck>::const_iterator i=LO.begin();i!=LO.end();++i)
   {
     // Option, die mit dem C. gespeichert werden müssen
     Tag &o=Opt.push_back(Tag("CheckOptions"));
     o.setAttr("Name", i->text);
     o.setBoolAttr("Wert", i->active);
   }
   const std::list<Optionen::st_Haus> &LH=optionen.getHausregeln();
   for(std::list<Optionen::st_Haus>::const_iterator i=LH.begin();i!=LH.end();++i)
   {
     Tag &o=Opt.push_back(Tag("Hausregeln"));
     o.setAttr("Name", i->text);
     o.setBoolAttr("Wert", i->active);
   }
   ts.write(datei);
   const_cast<class Abenteurer*>(this)->last_saved_time= time(0);
}


void Abenteurer::grundwerte_speichern(Tag &datei) const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
   Tag &Figur=datei.push_back(Tag("Figur"));
   Figur.setAttr_ne("Name", Name_Abenteurer());
   Figur.setAttr_ne("Spieler", Name_Spieler());
   Figur.setAttr_ne("Zeitpunkt", Version());
   Figur.setIntAttr("Grad", Grad());

   Tag &Typ=datei.push_back(Tag("Typ"));
   Typ.setAttr_ne("Spezies", Spezies()->Name());
   Typ.setAttr_ne("Geschlecht", Geschlecht_str());
   Typ.setAttr_ne("Abkürzung", Typ1()->Short());
   Typ.setAttr_ne("Abkürzung2", Typ2()->Short());
   Typ.setAttr_ne("Spezialgebiet", Spezialgebiet()->Name());
   Typ.setAttr_ne("Spezialisierung", Spezialisierung());
   Typ.setAttr_ne("Überleben",(*Ueberleben())->Name());
   Typ.setAttr_ne("Muttersprache",Muttersprache());
   Typ.setAttr("Stadt_Land", Stadt_Land_str());
   Typ.setAttr("Hand", Hand());

   Tag &BE=datei.push_back(Tag("Basiseigenschaften"));
   BE.setIntAttr("St", St());
   BE.setIntAttr("Gw", Gw());
   BE.setIntAttr("Gs", Gs());
   BE.setIntAttr("Ko", Ko());
   BE.setIntAttr("In", In());
   BE.setIntAttr("Zt", Zt());

   Tag &aE=datei.push_back(Tag("abgeleiteteEigenschaften"));
   aE.setIntAttr("Au", Au());
   aE.setIntAttr("pA", pA());
   aE.setIntAttr("Wk", Wk());
   aE.setIntAttr("Sb", Sb());
   aE.setIntAttr("B", B());
   aE.setIntAttr_nn("GG", GG());
   aE.setIntAttr_nn("SG", SG());

   Tag &Ew=datei.push_back(Tag("Erfolgswerte"));
   Ew.setIntAttr("Abwehr", Abwehr_wert());
   Ew.setIntAttr("Zaubern", Zaubern_wert());
   Ew.setIntAttr("ZauberResistenz", Resistenz());

   Tag &Gesundheit=datei.push_back(Tag("Gesundheit")); // schlechter Name ?
   Gesundheit.setIntAttr("LP", LP());
   Gesundheit.setIntAttr("AP", AP());

   Tag &Beschreibung=datei.push_back(Tag("Beschreibung")); // soziale?
   Beschreibung.setIntAttr("Alter", Alter());
   Beschreibung.setAttr("Gestalt", Gestalt());
   Beschreibung.setIntAttr("Gewicht", Gewicht());
   Beschreibung.setIntAttr("Größe", Groesse());
   Beschreibung.setAttr_ne("Stand", Stand());
   Beschreibung.setAttr_ne("Bezeichnung", Bezeichnung());
   Beschreibung.setAttr_ne("Merkmale", Merkmale());
   Beschreibung.setAttr_ne("Herkunft", Herkunft()->Name());
   Beschreibung.setAttr_ne("Glaube", Glaube());

   Tag &Verm=datei.push_back(Tag("Vermögen"));
   Verm.setIntAttr_nn("GS", Gold());
   Verm.setIntAttr_nn("SS", Silber());
   Verm.setIntAttr_nn("KS", Kupfer());

   Tag &Steigern=datei.push_back(Tag("Steigern"));
   Steigern.setIntAttr_nn("GFP", GFP());
   Steigern.setIntAttr_nn("AEP", AEP());
   Steigern.setIntAttr_nn("KEP", KEP());
   Steigern.setIntAttr_nn("ZEP", ZEP());
   if (get_Steigern_EP_Prozent()!=50)
      Steigern.setIntAttr("EPproGFP", get_Steigern_EP_Prozent());
   if (get_Grad_Basiswerte()!=Grad())
      Steigern.setIntAttr("Basiswerte", get_Grad_Basiswerte());
   if (Steigertage())
      Steigern.setAttr("benötigte_Tage", dtos(Steigertage()));

   Tag &Pp=Steigern.push_back(Tag("Praxispunkte"));
   Pp.setIntAttr_nn("Abwehr", AbwehrPP());
   Pp.setIntAttr_nn("Zaubern", ZaubernPP());
   Pp.setIntAttr_nn("Spezial", SpezialPP());
   Pp.setIntAttr_nn("Resistenz", ResistenzPP());

   Tag &Text=datei.push_back(Tag("Text",this->Beschreibung()));
   Text.setIntAttr_nn("Größe", BeschreibungPixSize());
   Text.setAttr_ne("Bild", BeschreibungPix());
}

void Abenteurer::save_ausruestung(Tag &datei,const std::list<AusruestungBaum> &AB) const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  for(AusruestungBaum::const_iterator i=AB.begin();i!=AB.end();++i)
   {  Tag &Ggs=datei.push_back(Tag("Gegenstand"));
      if (i->getAusruestung().Anzahl()!=1)
        Ggs.setIntAttr("Anzahl", i->getAusruestung().Anzahl());
      Ggs.setAttr("Bezeichnung", i->getAusruestung().Name());
      if (!i->getAusruestung().Beschreibung().empty())
        Ggs.setAttr("Beschreibung", i->getAusruestung().Beschreibung());
      if (!i->getAusruestung().Region().empty())
        Ggs.setAttr("Region", i->getAusruestung().Region());
      if (i->getAusruestung().Gewicht()!=0.0)
        Ggs.setFloatAttr("Gewicht", i->getAusruestung().Gewicht());
      if (i->getAusruestung().RuestungOhneGewicht())
        Ggs.setBoolAttr("RüstungOhneGewicht",i->getAusruestung().RuestungOhneGewicht());
      if (!i->getAusruestung().Material().empty())
        Ggs.setAttr("Besonderheit", i->getAusruestung().Material());
      if (i->getAusruestung().Sichtbar())
         Ggs.setBoolAttr("sichtbar", i->getAusruestung().Sichtbar());
      if (!i->empty())
         save_ausruestung(Ggs, i->getChildren());
   }
}

const std::string Abenteurer::Beruf() const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
 if(list_Beruf.empty()) return "";
 return (*(*(list_Beruf.begin())))->Name();
 abort(); //never get here
}

///////////////////////////////////////////////////////////////////////////////

bool Abenteurer::xml_import_stream(std::istream& datei)
{  return laden(datei);
}

bool Abenteurer::laden(std::istream& datei)
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
   reset();
   TagStream ts(datei);
   const Tag *data=0; // &ts.getContent();
   const Tag *top=ts.find("Midgard-Charakter");
   if (!top)
   {  data=ts.find("MidgardCG-data");
      if (!data) data=ts.find("MAGUS-data");
      if (data)
      {  top=data->find("Midgard-Charakter");
         if (!top) top=data->find("Midgard-Abenteurer");
      }
   }

  if (!top)
   {//  hauptfenster->InfoFenster->AppendShow("(Abenteurer in) Datei "//'"+Latin2Screen(datei)
    //  " nicht gefunden.");
      return false;
   }
   const int xml_version=top->getIntAttr("Version");
/*
   if (xml_version<3 || xml_version>10)
   {         hauptfenster->InfoFenster->AppendShow("XML Version "+itos(xml_version)
          +" wird noch nicht unterstützt");
   }
*/

   const Tag *Figur=top->find("Figur");
   const Tag *Typ=top->find("Typ");
   const Tag *Basiseigenschaften=top->find("Basiseigenschaften");
   const Tag *abgeleiteteEigenschaften=top->find("abgeleiteteEigenschaften");
   const Tag *Erfolgswerte=top->find("Erfolgswerte");
   const Tag *Resistenzen=top->find("Resistenzen");
   const Tag *Gesundheit=top->find("Gesundheit");
   const Tag *Beschreibung=top->find("Beschreibung");
   const Tag *Vermoegen=top->find("Vermoegen");
       if (!Vermoegen) Vermoegen=top->find("Vermögen");
   const Tag *Ruestung1=top->find("Ruestung");
       if (!Ruestung1) Ruestung1=top->find("Rüstung");
   const Tag *Ausruestung=top->find("Ausrüstung");
       if (!Ausruestung) Ausruestung=top->find("Fertigkeiten");
   Tag leer("");
       if (!Ausruestung) Ausruestung=&leer; // last Ressort
       if (!Ruestung1) Ruestung1=Ausruestung->find("Rüstung");
   const Tag *Ruestung2=Ausruestung->find("Rüstung2");
   const Tag *Fertigkeiten=top->find("Fertigkeiten");
   const Tag *Opt=top->find("Optionen");
   const Tag *Steigern=top->find("Steigern");
   const Tag *Praxispunkte=Steigern?Steigern->find("Praxispunkte"):0;

   setBasiswerte(Basiseigenschaften->getIntAttr("St"),
   	Basiseigenschaften->getIntAttr("Gw",abgeleiteteEigenschaften->getIntAttr("RW")),
   	Basiseigenschaften->getIntAttr("Gs",Basiseigenschaften->getIntAttr("Ge")),
   	Basiseigenschaften->getIntAttr("Ko"),
   	Basiseigenschaften->getIntAttr("In"),
   	Basiseigenschaften->getIntAttr("Zt"));
   setAu(abgeleiteteEigenschaften->getIntAttr("Au"));
   setpA(abgeleiteteEigenschaften->getIntAttr("pA"));
   setSb(abgeleiteteEigenschaften->getIntAttr("Sb"));
   setWk(abgeleiteteEigenschaften->getIntAttr("Wk"));
   setB(abgeleiteteEigenschaften->getIntAttr("B"));
   setLP(Gesundheit->getIntAttr("LP"));
   setAP(Gesundheit->getIntAttr("AP"));
   setZaubern_wert(Erfolgswerte->getIntAttr("Zaubern"));
   setAbwehr_wert(Erfolgswerte->getIntAttr("Abwehr"));
   setResistenz(Erfolgswerte->getIntAttr("ZauberResistenz",
  	!Resistenzen?0:Resistenzen->getIntAttr("Zauber",Resistenzen->getIntAttr("Resistenz",Resistenzen->getIntAttr("psy")))));
   if (Praxispunkte)
   {  setZaubernPP(Praxispunkte->getIntAttr("Zaubern"));
      setSpezialPP(Praxispunkte->getIntAttr("Spezial"));
      setAbwehrPP(Praxispunkte->getIntAttr("Abwehr"));
      setResistenzPP(Praxispunkte->getIntAttr("Resistenz"));    }
   else
   {  setZaubernPP(0);
      setAbwehrPP(0);
      setSpezialPP(0);
      setResistenzPP(0);
   }
   setHand(Typ->getAttr("Hand"));
   setGewicht(Beschreibung->getIntAttr("Gewicht"));
   setGroesse(Beschreibung->getIntAttr("Größe"));
   setGrad(Figur->getIntAttr("Grad"));
   setStand(Beschreibung->getAttr("Stand"));

   setAlter(Beschreibung->getIntAttr("Alter"));
   setGeschlecht(Typ->getAttr("Geschlecht","m"));
   std::string spezialgebiet=Typ->getAttr("Spezialgebiet");
   if(spezialgebiet!="")
     {
       cH_Spezialgebiet S(spezialgebiet);
       setSpezialgebiet(S);
     }
   std::string ueberleben=Typ->getAttr("Überleben");
   if(ueberleben!="")
     {
       H_MBEm M(cH_Fertigkeit(ueberleben,true));
       setUeberleben(M);
     }
   setMuttersprache(Typ->getAttr("Muttersprache"));
   setSpezialisierung(Typ->getAttr("Spezialisierung"));
   setBezeichnung(Beschreibung->getAttr("Bezeichnung"));
   setMerkmale(Beschreibung->getAttr("Merkmale"));
   setHerkunft(cH_Land(Beschreibung->getAttr("Herkunft"),true));
   setGlaube(Beschreibung->getAttr("Glaube"));
   setNamen(Figur->getAttr("Name"),Figur->getAttr("Spieler"),Figur->getAttr("Zeitpunkt"));
   setGFP(Steigern ? Steigern->getIntAttr("GFP", top->getInt("GFP")) : top->getInt("GFP"));
   setSteigertage(Steigern ? Steigern->getFloatAttr("benötigte_Tage", top->getInt("benötigte_Tage")) : top->getInt("benötigte_Tage"));
   setGG(abgeleiteteEigenschaften->getIntAttr("GG"));
   setSG(abgeleiteteEigenschaften->getIntAttr("SG"));
   const Tag *Text=top->find("Text");
   if (!Text) Text=top; // to avoid testing below
   setBeschreibung(Text->Value());
   setBeschreibungPix(top->getString("TextPix",Text->getAttr("Bild")));
   setBeschreibungPixSize(top->getInt("TextPixSize",Text->getIntAttr("Größe")));
   if (Vermoegen)
      setGeld(Vermoegen->getIntAttr("GS"),Vermoegen->getIntAttr("SS"),Vermoegen->getIntAttr("KS"));
//   clearRuestung();
   if (Ruestung1) setRuestung1(Ruestung1->Value(),true);
   else setRuestung1("OR",true);
   if (Ruestung2) setRuestung2(Ruestung2->Value(),true);
   else setRuestung2("OR",true);
   setSpezies(cH_Spezies(Typ->getAttr("Spezies","Mensch"),true));
   if (Steigern)       setEP(Steigern->getIntAttr("AEP"),Steigern->getIntAttr("KEP"),Steigern->getIntAttr("ZEP"));
   else
      setEP(0,0,0);
   setStadtLand(Typ->getAttr("Stadt_Land","Stadt"));
   if (Steigern)
   { int fpanteil2=Steigern->getIntAttr("EPproGFP",50);
     fpanteil=fpanteil2;
     goldanteil=100-fpanteil2;
     set_Grad_Basiswerte(Steigern->getIntAttr("Basiswerte",Grad()));
   }
   else
   { fpanteil=goldanteil=50;
     set_Grad_Basiswerte(Grad());
   }

//   this->Typ1()=cH_Typen(Typ->getAttr("Abkürzung"),true);
   setTyp1(cH_Typen(Typ->getAttr("Abkürzung"),true));

   { std::string s = Typ->getAttr("Abkürzung2");
     if(s!="") //this->Typ2=cH_Typen(s,true);
         setTyp2(cH_Typen(Typ->getAttr("Abkürzung2"),true));
   }

   load_fertigkeiten(Fertigkeiten,Ausruestung,xml_version);
   load_regionen_optionen(Opt,xml_version);
   getBesitz().clear();
   load_ausruestung(Ausruestung,&(getBesitz()));
   FOR_EACH_CONST_TAG_OF(i,*top,"Von Spruchrolle gelernt")
     list_Gelernt_von_Spruchrolle.push_back(i->Value());
   return true;
}

void Abenteurer::load_ausruestung(const Tag *tag, AusruestungBaum *AB)
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  FOR_EACH_CONST_TAG_OF(i,*tag,"Gegenstand")
   {  cH_Preise(i->getAttr("Bezeichnung"),true);
      AusruestungBaum *A = &AB->push_back(
      	Ausruestung(i->getIntAttr("Anzahl",1),
      	         i->getAttr("Bezeichnung"),i->getFloatAttr("Gewicht"),
      	         i->getAttr("Besonderheit"),
   					i->getAttr("Region"),i->getBoolAttr("sichtbar"),
   					i->getBoolAttr("RüstungOhneGewicht"),
   					i->getAttr("Beschreibung")));
      A->setParent(AB);
      load_ausruestung(&*i,A);
   }
}

void Abenteurer::load_fertigkeiten(const Tag *tag, const Tag *waffen_b, int xml_version)
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
    FOR_EACH_CONST_TAG(i,*tag)
    {
      const std::string sart=i->Type();
      if(sart=="Sinn")
        {
         setSinn(i->getAttr("Bezeichnung"),i->getIntAttr("Wert"));
        }
      else if(sart=="Beruf")
        {
         cH_MBE beruf(cH_Beruf(i->getAttr("Bezeichnung"),true));
         H_MBEm B(beruf);
         B->setErfolgswert(i->getIntAttr("Wert"));
         List_Beruf().push_back(B);
        }
      else if(sart=="ang-Fertigkeit" || sart=="ang.Fertigkeit")
       {
         cH_MBE fert_an(cH_Fertigkeit_angeborene(i->getAttr("Bezeichnung"),true));
         H_MBEm F(fert_an);
         F->setErfolgswert(i->getIntAttr("Wert"));
         List_Fertigkeit_ang().push_back(F);
       }
      else if(sart=="Fertigkeit")
       {
         cH_MBE fert(cH_Fertigkeit(i->getAttr("Bezeichnung"),true));
         H_MBEm F(fert);
         F->setErfolgswert(i->getIntAttr("Wert"));
         F->setPraxispunkte(i->getIntAttr("Praxispunkte"));
         MBE_T::eZusatz zus=cH_Fertigkeit(fert)->ZusatzEnum(getVTyp());
         if(zus)
         {  // fert=new Fertigkeit(*cH_Fertigkeit(fert));
            // create Zusatz if not already there
            if(zus==MBE_T::ZLand) cH_Land(i->getAttr("Zusatz"),true);
            else if(zus==MBE_T::ZWaffe) cH_Waffe(i->getAttr("Zusatz"),true);
            F->setZusatz(i->getAttr("Zusatz"));
         }
         F->loadWieGelernt(*i);
         List_Fertigkeit().push_back(F);
       }
      else if(sart=="Waffe")
        {
         cH_MBE waffe(cH_Waffe(i->getAttr("Bezeichnung"),true));
         H_MBEm W(waffe);
         W->setErfolgswert(i->getIntAttr("Wert"));
         W->setPraxispunkte(i->getIntAttr("Praxispunkte"));
         W->loadWieGelernt(*i);
         List_Waffen().push_back(W);
        }
#if 0 // andere Liste
      if(sart=="Besitz_W")
#endif
      else if(sart=="Zauber")
        {
         cH_MBE zauber(cH_Zauber(i->getAttr("Bezeichnung"),true));
         H_MBEm Z(zauber);
         if(zauber->ZusatzEnum(getVTyp()))
          { // zauber=new Zauber(*cH_Zauber(zauber));
            Z->setZusatz(i->getAttr("Zusatz"));
          }
         List_Zauber().push_back(Z);
        }
      else if(sart=="Zauberwerk")
        {
          cH_MBE zauberwerk(cH_Zauberwerk(i->getAttr("Bezeichnung"),
                      i->getAttr("Art"),i->getAttr("Stufe"),true));
          H_MBEm Z(zauberwerk);
          List_Zauberwerk().push_back(Z);
        }
      else if(sart=="KiDo")
        {
          cH_MBE kido(cH_KiDo(i->getAttr("Bezeichnung"),true));
          H_MBEm K(kido);
          List_Kido().push_back(K) ;
        }
      else if(sart=="Grundkenntnis")
        {  std::string bez=i->getAttr("Bezeichnung");
           if (bez=="ESchwert") bez="Einhandschwert";
           else if (bez=="ZSchwert") bez="Zweihandschwert";
           else if (bez=="ESchlagwaffe") bez="Einhandschlagwaffe";
           else if (bez=="ZSchlagwaffe") bez="Zweihandschlagwaffe";
           else if (bez=="Schilde") bez="Schild";
           cH_MBE grund(cH_WaffeGrund(bez,true));
           H_MBEm G(grund);
           List_WaffenGrund().push_back(G);
        }
      else if(sart=="Sprache")
        {
         cH_MBE sprache(cH_Sprache(i->getAttr("Bezeichnung"),true));
         H_MBEm S(sprache);
         int wert=i->getIntAttr("Wert");
         if (xml_version<8)          {  switch (wert)
            {  case 1: wert=4; break;
               case 2: wert=6; break;
               case 3: wert=10; break;
               case 4: wert=13; break;
            }
         }
         S->setErfolgswert(wert);
         S->setPraxispunkte(i->getIntAttr("Praxispunkte"));
         List_Sprache().push_back(S);
        }
      else if(sart=="Urschrift")
        {
         cH_MBE schrift(cH_Schrift(i->getAttr("Bezeichnung"),true));
         H_MBEm S(schrift);
         int wert=i->getIntAttr("Wert");
         if (xml_version<8 && !wert) wert=12;
         S->setErfolgswert(wert);
         S->setPraxispunkte(i->getIntAttr("Praxispunkte"));
         List_Schrift().push_back(S);
        }
// demnächst weg (geändert seit 0.8.10)
      else if(sart=="Optionen")
        {
         try{
           optionen.setOptionCheck(i->getAttr("Name"),i->getBoolAttr("Wert"));
         }
         catch (const NotFound &e)
         { Ausgabe(Ausgabe::Error,e);  }
        }
      else if(sart=="Region")
        {
         try
         {  cH_Region R(i->getAttr("Name",i->getAttr("Region")));
            regionen[R]=true;
         }
// bis hier
         catch (const NotFound &e)
         {  Ausgabe(Ausgabe::Error,e);}
        }
    }
    FOR_EACH_CONST_TAG_OF(i,*waffen_b,"Waffe")
    {   H_WaffeBesitz WB=new WaffeBesitz(Datenbank.WaffeVonBezeichnung(i->getAttr("Bezeichnung")),
                        i->getAttr("Bezeichnung"),
                        i->getIntAttr("AngriffVerteidigung_Bonus"),
                        i->getIntAttr("SchadenLebenspunkte_Bonus"),
                        i->Value(),
                        i->getAttr("Region"));
        List_Waffen_besitz().push_back(WB);
    }
}

void Abenteurer::load_regionen_optionen(const Tag *tag, int xml_version)
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
    for(std::map<cH_Region,Model_copyable<bool> >::iterator i=regionen.begin();i!=regionen.end();++i)
      {   i->second=false;  }
    if(!tag) return;
    FOR_EACH_CONST_TAG(i,*tag)
    {
      const std::string sart=i->Type();
      if(sart=="CheckOptions")
        {
         try{
           optionen.setOptionCheck(i->getAttr("Name"),i->getBoolAttr("Wert"));
         }
         catch (const NotFound &e)
         {Ausgabe(Ausgabe::Error,e);}
        }
      else if(sart=="Hausregeln")
        {
         try{
           optionen.setHausregeln(i->getAttr("Name"),i->getBoolAttr("Wert"));
         }
         catch (const NotFound &e)
         {Ausgabe(Ausgabe::Error,e);}
        }
      else if(sart=="Region")
        {
         try
         {  cH_Region R(i->getAttr("Name",i->getAttr("Region")));
            regionen[R]=true;
         }
         catch (const NotFound &e)
         {Ausgabe(Ausgabe::Error,e);}
        }
   }
}


void Abenteurer::move_element(std::list<H_MBEm>& von,
                                       std::list<H_MBEm>& nach,
                                       const H_MBEm& MBE)
{
 ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
 for (std::list<H_MBEm>::iterator i=von.begin();i!= von.end();++i)
  {
    if(*i==MBE)
        {
          (*i)->setErfolgswert(MBE->Erfolgswert());
          nach.splice(nach.begin(),von,i);
          break;
        }
  }
}


void Abenteurer::remove_WaffenGrund()
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
reloop:
  for(std::list<H_MBEm>::iterator i=list_WaffenGrund.begin();i!=list_WaffenGrund.end();++i)
   {
     bool remove=true;
     for(std::list<H_MBEm>::const_iterator j=list_Waffen.begin();j!=list_Waffen.end();++j)
      {
        if(cH_Waffe((*j)->getMBE())->Grundkenntnis()==(*(*i))->Name())
            {remove=false; break;}
      }
     if(remove) {list_WaffenGrund.remove(*i); goto reloop;}
   }
}

// wozu??? um es passend einzuhängen
std::list<H_MBEm> &Abenteurer::get_known_list(const MBE_T::MBEE was)
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  switch (was) {
     case MBE_T::FERTIGKEIT: return List_Fertigkeit(); break;
     case MBE_T::WAFFE: return List_Waffen(); break;
     case MBE_T::SPRACHE: return List_Sprache(); break;
     case MBE_T::SCHRIFT: return List_Schrift(); break;
     case MBE_T::WAFFEGRUND: return List_WaffenGrund(); break;
     case MBE_T::ZAUBER: return List_Zauber(); break;
     case MBE_T::ZAUBERWERK:return List_Zauberwerk(); break;
     case MBE_T::KIDO: return List_Kido(); break;
     default: assert(!"never get here\n");
   }
}

std::list<H_MBEm> const &Abenteurer::get_known_list(const MBE_T::MBEE was) const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  switch (was) {
     case MBE_T::FERTIGKEIT: return List_Fertigkeit(); break;
     case MBE_T::WAFFE: return List_Waffen(); break;
     case MBE_T::SPRACHE: return List_Sprache(); break;
     case MBE_T::SCHRIFT: return List_Schrift(); break;
     case MBE_T::WAFFEGRUND: return List_WaffenGrund(); break;
     case MBE_T::ZAUBER: return List_Zauber(); break;
     case MBE_T::ZAUBERWERK:return List_Zauberwerk(); break;
     case MBE_T::KIDO: return List_Kido(); break;
     default: assert(!"never get here\n");
   }
}


std::list<H_MBEm> &Abenteurer::get_known_list(const H_MBEm &MBE)
{ ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
  return getList((*MBE).What());
}

std::list<H_MBEm> &Abenteurer::getList(MBE_T::MBEE was)
{ switch (was)
  { case MBE_T::FERTIGKEIT: return List_Fertigkeit();
    case MBE_T::WAFFE: return List_Waffen();
    case MBE_T::WAFFEGRUND: return List_WaffenGrund();
    case MBE_T::ZAUBER: return List_Zauber();
    case MBE_T::ZAUBERWERK: return List_Zauberwerk();
    case MBE_T::KIDO: return List_Kido();
    case MBE_T::SPRACHE: return List_Sprache();
    case MBE_T::SCHRIFT: return List_Schrift();
    default: assert(!"Abenteurer::getList: invalid arg");
      abort(); // silence gcc
  }
}

std::string Abenteurer::Ruestung_B_Verlust(bool ueberlast_beruecksichtigen) const
{
  ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__);
 std::string s;
 int R0;
 int R1;

 bool ew; // Erfolgswerte um zwei reduzieren
 if(ueberlast_beruecksichtigen)
  {
    R0 = Ruestung(0)->B_Verlust(getUeberlast(),*this,ew);
    R1 = Ruestung(1)->B_Verlust(getUeberlast(),*this,ew);
  }
 else
  {
    R0 = Ruestung(0)->B_Verlust(0,*this,ew);
    R1 = Ruestung(1)->B_Verlust(0,*this,ew);
  }

 int b=B();

 if     (R0&&!R1) s="\\scriptsize("+itos(b-R0)+")";
 else if(R1&&!R0) s="\\scriptsize("+itos(b-R1)+")";
 else if(R0&&R1)  s="\\tiny("+itos((b-R0<0)?0:b-R0)+","+itos((b-R1<0)?0:b-R1)+")";

 if(s!="") return "_{"+s+"}";
 return "";
}

bool Abenteurer::getRegion(const cH_Region &r) const
{  regionen_t::const_iterator i=regionen.find(r);
   if (i==regionen.end()) return false;
   return i->second.Value();
}

void Abenteurer::reset()
{  list_Beruf.clear();
             list_Fertigkeit_ang.clear();list_Fertigkeit.clear();
             list_WaffenGrund.clear();list_Waffen.clear();
             list_Waffen_besitz.clear();list_Zauber.clear();
             list_Zauberwerk.clear();list_Kido.clear();
             list_Sprache.clear();list_Schrift.clear();
             Grundwerte::reset();
             setStandardAusruestung();
}

   // true: funktioniert
bool Abenteurer::Steigern(H_MBEm &MBE)
{ return steigere(MBE);
//  return true;
}

// eine Fertigkeit mit FP etc. erlernen, Kosten werden noch abgezogen
H_MBEm Abenteurer::Erlernen(cH_MBE const&MBE, int bonus)
{ return neu_lernen(MBE, bonus);
}

// all in one function
H_MBEm Abenteurer::Erlernen(H_LSE const& lse)
{
	 if(((*lse).What()==MBE_T::FERTIGKEIT || (*lse).What()==MBE_T::WAFFE)
	   && !(*lse)->Voraussetzung(*this,false))
	  {
	    Ausgabe(Ausgabe::Error,"Erst muss "+(*lse)->Voraussetzung()+" gelernt werden");
	    return H_MBEm::null();
	  }

     if(Typ1()->Kultwaffe() && List_Waffen().empty() && lse->getListe()==MBEm::wie_erlernt::separat
	    		&& (*lse)->What()==MBE_T::WAFFE && lse->Lernpunkte()>getLernpunkte().getLernpunkte().Waffen())
       {
         // Für eine Kultwaffe werden fehlende Lernpunkte geschenkt
         getLernpunkte().getLernpunkte().setWaffen(lse->Lernpunkte());
         Ausgabe(Ausgabe::Log,(*lse)->Name()+" als Kultwaffe gewählt");
       }

     if ((lse->getListe()==MBEm::wie_erlernt::fachkenntnis
	    		&& getLernpunkte().getLernpunkte().Fach() < lse->Lernpunkte())
	    	|| (lse->getListe()==MBEm::wie_erlernt::allgemeinwissen
	    		&& getLernpunkte().getLernpunkte().Allgemein() < lse->Lernpunkte())
	    	|| (lse->getListe()==MBEm::wie_erlernt::ungewoehnlich
	    		&& getLernpunkte().getLernpunkte().Unge() < lse->Lernpunkte())
	    	|| (lse->getListe()==MBEm::wie_erlernt::separat
	    		&& (*lse)->What()==MBE_T::WAFFE
	    		&& getLernpunkte().getLernpunkte().Waffen() < lse->Lernpunkte())
	    	|| (lse->getListe()==MBEm::wie_erlernt::separat
		    	&& (*lse)->What()==MBE_T::ZAUBER
		    	&& getLernpunkte().getLernpunkte().Zauber() < lse->Lernpunkte()))
	    {
	    	Ausgabe(Ausgabe::Error,"Nicht genug Lernpunkte");
	    	return H_MBEm::null();
	    }

    MBEm::wie_erlernt w;
	w.wie= MBEm::wie_erlernt::lernen;
	w.lernen_detail.liste= lse->getListe();
	w.lernen_detail.lp= lse->Lernpunkte();
	bool nachbarland= w.lernen_detail.liste==MBEm::wie_erlernt::allgemeinwissen;
    H_MBEm M=lse->lernen(w,LSE_T::lernen_param(*this, nachbarland));
    if (!M) return M; // Fehler
    get_known_list((*M).What()).push_back(M);
    if((*M)->What()==MBE_T::FERTIGKEIT)
    {
      std::string::size_type st = (*lse)->Name().find("KiDo-Technik");
      if(st!=std::string::npos)
      { ++getLernpunkte().MaxKido();
//#warning was ist das?
//       list_FertigkeitZusaetze.push_back((*lse)->Name());
      }
      if((*lse)->Name()=="KiDo" && Typ1()->Short()=="Kd")
        getLernpunkte().MaxKido()+=2;
#if 0
      if(getLernpunkte().MaxKido()>0 && H_MBEm(cH_Fertigkeit("KiDo"))->ist_gelernt(List_Fertigkeit()))
         zeige_werte();
#endif
   }
    if((*M)->What()==MBE_T::WAFFE)
    { cH_Waffe w(M->getMBE());
      if (!w->Grundkenntnis().empty())
      {
//  #warning besser vorher nachsehen statt hinterher löschen
        List_WaffenGrund().push_back(H_MBEm(cH_WaffeGrund(w->Grundkenntnis())));

        if(!w->ZweiteGrundkenntnis().empty() && w->Min_St_Einhand(*this))
           List_WaffenGrund().push_back(H_MBEm(cH_WaffeGrund(w->ZweiteGrundkenntnis())));

        List_WaffenGrund().sort(H_MBEm::sort(H_MBEm::sort::NAME));
        List_WaffenGrund().unique();
      }
      else Ausgabe(Ausgabe::Warning,"Waffe "+w->Name()+" hat keine Grundkenntnisse");
    }
    if(lse->getListe()==MBEm::wie_erlernt::fachkenntnis)
       getLernpunkte().getLernpunkte().addFach(-lse->Lernpunkte());
    else if(lse->getListe()==MBEm::wie_erlernt::allgemeinwissen)
       getLernpunkte().getLernpunkte().addAllgemein(-lse->Lernpunkte());
    else if(lse->getListe()==MBEm::wie_erlernt::ungewoehnlich)
       getLernpunkte().getLernpunkte().addUnge(-lse->Lernpunkte());
    else if ((*lse)->What()==MBE_T::WAFFE)
       getLernpunkte().getLernpunkte().addWaffen(-lse->Lernpunkte());
    else if ((*lse)->What()==MBE_T::ZAUBER)
       getLernpunkte().getLernpunkte().addZauber(-lse->Lernpunkte());
    return M;
}

   // false: Fehlgeschlagen
bool Abenteurer::ReduzierenVerlernen(H_MBEm &MBE, bool &verlernt)
{ verlernt=false;
  if (!MBE->AlleSteigern().empty())
  {  MBEm_T::wie_gesteigert wie=MBE->AlleSteigern().back();
	// Kopie anlegen, damit ältere Undo Schritte nicht verändert werden
	if (MBE->What()!=MBE_T::RESISTENZ_UND_CO)
	{ H_MBEm old=MBE;
	  MBE=MBE.dup();
	  std::list<H_MBEm> &L=getList(MBE->What());
	  std::replace(L.begin(),L.end(),old,MBE);
	}
     MBE->AlleSteigern().erase(--MBE->AlleSteigern().end());
     MBE->setErfolgswert(wie.alter_wert);
     switch (wie.wie)
     { case MBEm_T::wie_erlernt::lernen:
    	 assert(!"Fähigkeiten kann man nicht per Lernschema steigern?");
       case MBEm_T::wie_erlernt::beruf:
       case MBEm_T::wie_erlernt::angeboren:
    	 break;
       case MBEm_T::wie_erlernt::praxis:
    	 MBE->addPraxispunkte(wie.praxis_detail.praxispunkte);
    	 addKEP(wie.praxis_detail.fp.kep);
    	 addAEP(wie.praxis_detail.fp.aep);
    	 addZEP(wie.praxis_detail.fp.zep);
    	 addGFP(-wie.praxis_detail.fp.Summe()); // falsch
    	 break;
       case MBEm_T::wie_erlernt::unterweisung:
       case MBEm_T::wie_erlernt::selbststudium:
    	 addGold(wie.unterweisung_detail.gs);
    	 addKEP(wie.unterweisung_detail.fp.kep);
    	 addAEP(wie.unterweisung_detail.fp.aep);
    	 addZEP(wie.unterweisung_detail.fp.zep);
    	 // if(!optionen.HausregelCheck(Optionen::Gold).active) gold_k*=10;
    	 // set_lernzeit(-ep_k,H_MBEm(0));
    	 addGFP(-wie.unterweisung_detail.fp.Summe());
    	 break;
       case MBEm_T::wie_erlernt::spruchrolle:
       default:
    	 assert(!"interessante Art zu steigern entdeckt");
     }
     return true; // reduktion hat funktioniert
  }
  else
  { verlernt=true;
    MBEm_T::wie_erlernt wie=MBE->getLernArt();
    getList((*MBE).What()).remove(MBE);
	 if ((*MBE).What()==MBE_T::WAFFEBESITZ)
	 {
         cH_Waffe W(MBE->getMBE());
         std::string art=W->Art2();
#if 0
         if(art=="E" || art=="W" || art=="V") waffebesitzlernen.add_EWaffe(1);
         else waffebesitzlernen.add_AWaffe(1);
#endif
	 }
	 else switch (wie.wie)
    { case MBEm_T::wie_erlernt::lernen:
    	 switch (wie.lernen_detail.liste)
    	 { case MBEm_T::wie_erlernt::allgemeinwissen:
    		 getLernpunkte().getLernpunkte().addAllgemein(wie.lernen_detail.lp);
    		 break;
    	   case MBEm_T::wie_erlernt::fachkenntnis:
    	     getLernpunkte().getLernpunkte().addFach(wie.lernen_detail.lp);
    	     break;
    	   case MBEm_T::wie_erlernt::ungewoehnlich:
    	     getLernpunkte().getLernpunkte().addUnge(wie.lernen_detail.lp);
    	     break;
    	   case MBEm_T::wie_erlernt::separat:
    		 switch ((*MBE).What())
    		 { case MBE_T::WAFFE:
    			 getLernpunkte().getLernpunkte().addWaffen(wie.lernen_detail.lp);
    			 break;
        	   case MBE_T::ZAUBER:
        		 getLernpunkte().getLernpunkte().addZauber(wie.lernen_detail.lp);
        		 break;
        	   case MBE_T::KIDO:
                 ++getLernpunkte().MaxKido();
        		 break;
        	   default:
        		 assert(!"ungewöhnliche Art zu lernen");
        		 break;
    		 }
    	     break;
    	 }
    	 if ((*MBE).What()==MBE_T::WAFFE)
    	 {
    		// nachsehen ob noch andere Waffe gleiche Grundkenntnis hat, sonst Grundkenntnis löschen
    		 remove_WaffenGrund();
    	 }
    	 // zusätzliche KiDo Technik
         if((*MBE)->Name().find("KiDo-Technik")!=std::string::npos)
        	 --getLernpunkte().MaxKido();
      	 break;
      case MBEm_T::wie_erlernt::beruf:
      case MBEm_T::wie_erlernt::angeboren:
      	 break;
      case MBEm_T::wie_erlernt::praxis:
      	 MBE->addPraxispunkte(wie.praxis_detail.praxispunkte);
      	 addKEP(wie.praxis_detail.fp.kep);
      	 addAEP(wie.praxis_detail.fp.aep);
      	 addZEP(wie.praxis_detail.fp.zep);
      	 addGFP(-wie.praxis_detail.fp.Summe());
      	 break;
      case MBEm_T::wie_erlernt::unterweisung:
      case MBEm_T::wie_erlernt::selbststudium:
      	 addGold(wie.unterweisung_detail.gs);
      	 addKEP(wie.unterweisung_detail.fp.kep);
      	 addAEP(wie.unterweisung_detail.fp.aep);
      	 addZEP(wie.unterweisung_detail.fp.zep);
      	 addGFP(-wie.unterweisung_detail.fp.Summe());
      	 break;
      case MBEm_T::wie_erlernt::spruchrolle:
    	  // GFP+Zeit abziehen
         default:
      	 assert(!"interessante Art zu steigern entdeckt");
    }
    return true; // reduktion hat funktioniert
  }
  return false;
#if 0 // alte Logik (ohne Geschichte)
  if (MBE->Reduzieren(*this))
    reduziere(MBE);
  else if ((*MBE).What()==MBE_T::WAFFE)
  { // Waffen kann man nicht verlernen
    verlernt=true;
  }
  else if (MBE->Verlernen(*this))
  { verlerne(MBE);
    verlernt=true;
  }
  else return false;
#endif
}

bool Abenteurer::SpruchVonSpruchrolleGelernt(const std::string &zauber)
{ return find(list_Gelernt_von_Spruchrolle.begin(),list_Gelernt_von_Spruchrolle.end(),zauber)!=list_Gelernt_von_Spruchrolle.end() ; }

bool Abenteurer::sort_universell::operator()(const st_universell &x,const st_universell &y) const
{ return (*(x.mbe))->Name() < (*(y.mbe))->Name() ;
}

unsigned Abenteurer::VerfuegbareFP() const
{ int EP=AEP()+KEP()+ZEP();
  int GS=Gold();
  if (getOptionen().HausregelCheck(Optionen::Gold).active) GS*=10;
  if (fpanteil.Value()>0) EP=EP*100/fpanteil.Value();
  else EP=1000000;
  if (goldanteil.Value()>0) GS=GS*10/goldanteil.Value();
  else GS=1000000;
  if (GS<EP) EP=GS;
  if (EP==1000000) EP=0;
  return EP;
}

H_MBEm Abenteurer::get_known(MBE_T::MBEE was, std::string const& name, std::string const& zusatz)
{ std::list<H_MBEm> l=get_known_list(was);
  for (std::list<H_MBEm>::const_iterator i=l.begin();i!=l.end();++i)
  { if ((**i)->Name()==name)
    { if ((**i)->ZusatzEnum(getVTyp())!=MBE_T::ZNone)
      { if ((*i)->Zusatz()==zusatz) return *i;
      }
      else return *i;
    }
  }
  return H_MBEm::null();
}

// verwendet von LernListen::get_steigern_MBEm, daher muss für alle unlernbaren
// Dinge null zurückgegeben werden
bool Abenteurer::lernbar(cH_MBE const& was) const
{ if (was->What()==MBE_T::FERTIGKEIT &&
      (was->Name()=="Sprache" || was->Name()=="Schreiben" || was->Name()=="KiDo-Technik"))
        return false;
  if (Spezies()->istVerboten(was)) return false;
  if (was->Name()=="Zaubern" &&  is_mage() ) return false;
  if (!was->ist_lernbar(getVTyp(),was->get_MapTyp())) return false;
  if (!LernListen::region_check(*this,was->Region())) return false;
  if (!LernListen::nsc_check(*this,was->NSC_only())) return false;
  H_MBEm MBEm(was);
  if (was->ZusatzEnum(getVTyp())==MBE_T::ZNone
      && MBEm->ist_gelernt(get_known_list(was->What())))
    return false; // schon erlernt
  switch(was->What())
  {
   case MBE_T::FERTIGKEIT:
      if (!was->Voraussetzung(*this)) return false;
//      MBEm->setErfolgswert(was->Anfangswert());
      break;
   case MBE_T::WAFFE:
    { const cH_Waffe w(was);
      if (!w->Grundkenntnis_vorhanden( List_WaffenGrund())) return false;
      if (!was->Voraussetzung(*this)) return false;
//      if (w->Art()=="Verteidigung") MBEm->setErfolgswert(1);
//      else MBEm->setErfolgswert(4);
      break;
     }
   case MBE_T::WAFFEGRUND:
      if(!cH_WaffeGrund(was)->is_sinnvoll(Datenbank.Waffe,*this)) return false;
      break;
   case MBE_T::SPRACHE:
//      MBEm->setErfolgswert(cH_Fertigkeit("Sprache")->Anfangswert());
      break;
   case MBE_T::SCHRIFT:
      if (!cH_Schrift(was)->kann_Sprache( List_Sprache())) return false;
//      MBEm->setErfolgswert(cH_Fertigkeit("Schreiben")->Anfangswert());
      break;
   case MBE_T::ZAUBER:
   case MBE_T::ZAUBERWERK:
      break;
   default : assert(!"get here");
  }
  return true;
}

//#warning Spezialwaffe wählen in libmagus
#if 0
if(togglebutton_spezialwaffe->get_active() )
 {
  cH_Waffe W(MBE->getMBE());
  if(W->Verteidigung())
    { Ausgabe(Ausgabe::Error,"Eine Verteidingungswaffe kann keine Spezialwaffe werden."); hauptfenster->getChar()->cancel_undo(); return;}
  else if(W->Reichweite().empty() && (*MBE).AnzahlLernpunkte()>1 )
    { Ausgabe(Ausgabe::Error,"Nahkampfwaffen können nur dann eine Spezialwaffe werden wenn sie max. 1 Lernpunkte gekostet haben");hauptfenster->getChar()->cancel_undo();return;}
  else if(!W->Reichweite().empty() && (*MBE).AnzahlLernpunkte()>2 )
    { Ausgabe(Ausgabe::Error,"Fernkampfwaffen können nur dann eine Spezialwaffe werden wenn sie max. 2 Lernpunkte gekostet haben");hauptfenster->getChar()->cancel_undo();return;}
  else
    {
     hauptfenster->getAben().setSpezialisierung((*MBE)->Name());
     Waffe::setSpezialWaffe(hauptfenster->getAben().Spezialisierung(),hauptfenster->getAben().List_Waffen());
     togglebutton_spezialwaffe->set_active(false);
     hauptfenster->getChar().getWizard().done(Wizard::SPEZIALWAFFE,hauptfenster->getAben());
    }
 }
#endif
bool Abenteurer::SpezialwaffeWaehlen()
{
	  if (!Typ1()->Spezialwaffe() && !Typ2()->Spezialwaffe()) return false;
	  std::list<H_MBEm> L=List_Waffen();
	  std::vector<MBE_T::st_zusatz> V;
	  for(std::list<H_MBEm>::const_iterator i=L.begin();i!=L.end();++i)
	   {
		  cH_Waffe mbe=cH_Waffe((*i)->getMBE());
	     if(mbe->Verteidigung()) continue;
	     if(mbe->Reichweite().empty() && (*i)->AnzahlLernpunkte() > 1) continue;
	     if(!mbe->Reichweite().empty() && (*i)->AnzahlLernpunkte() > 2) continue;
	     V.push_back(MBE_T::st_zusatz(mbe));
	   }
	  if(V.empty()) return false;
	  MBE_T::st_zusatz res=LSE_T::Auswaehlen(V, "Spezialwaffe wählen");
	  if (!res.mbe) return false;
	  setSpezialisierung(res.name);
	  Waffe::setSpezialWaffe(Spezialisierung(),List_Waffen());
	  return true;
}

bool Abenteurer::SpezialgebietWaehlen()
{
	  if (!Typ1()->Spezialgebiet() && !Typ2()->Spezialgebiet()) return false;
	  std::vector<std::string> V=LernListen::getSpezialgebiet(*this);
	  if(V.empty()) return false;
	  std::vector<MBE_T::st_zusatz> V2;
	  for (std::vector<std::string>::const_iterator i=V.begin();i!=V.end();++i)
		  V2.push_back(*i);
	  MBE_T::st_zusatz res=LSE_T::Auswaehlen(V2, "Spezialgebiet wählen");
	  if (!res.mbe) return false;
	  setSpezialgebiet(cH_Spezialgebiet(res.name));
	  return true;
}

bool Abenteurer::HerkunftWaehlen()
{
	  H_MBEm Mdummy(NULL);
	  //lernen_zusatz_titel(was,MBE);
	  std::vector<MBE_T::st_zusatz> zusatzliste;
	  std::vector<std::pair<cH_Land,bool> > L=LernListen::getHerkunft(*this);
	  for(std::vector<std::pair<cH_Land,bool> >::const_iterator i=L.begin();i!=L.end();++i)
	  {
	     MBE_T::st_zusatz zs(i->first,i->second);
	     zusatzliste.push_back(zs);
	  }
	  MBE_T::st_zusatz res=LSE_T::Auswaehlen(zusatzliste, "Herkunftsland wählen");
	  cH_Land land=res.land.cast_dynamic<Land const>();
	  if (!land->Name().empty())
	  {
		  setHerkunft(land->Name());
		  // Region anschalten
		  if (!land->Name().empty())
		      getRegion(cH_Region::getRegionfromAbk(land->Region()))=true;
		  return true;
	  }
	  return false;
}

bool Abenteurer::MutterspracheWaehlen()
{
	  std::list<H_LSE> V_=LernListen::getMBEm(*this,LernListen::MutterSprache);
	  std::vector<MBE_T::st_zusatz> V;
	  for(std::list<H_LSE>::const_iterator i=V_.begin();i!=V_.end();++i)
	   {
	      V.push_back(MBE_T::st_zusatz((*i)->getMBE(),(*i)->Erlaubt()));
	   }
	  MBE_T::st_zusatz res=LSE_T::Auswaehlen(V, "Muttersprache wählen ("+Herkunft()->Name()+")");
	  if (!res.mbe) return false;
	  MBEm::wie_erlernt wie;
	  wie.wie=MBEm::wie_erlernt::angeboren;
	  H_MBEm spr=LSE_T::lernen(res.mbe,wie,*this);
	  spr->setErfolgswert(Sprache::ErfolgswertMuttersprache(In(),cH_Fertigkeit("Sprache")->AttributBonus(*this)));
	  List_Sprache().push_back(spr);
	  setMuttersprache((*spr)->Name());
	  return true;
}

bool Abenteurer::UeberlebenWaehlen()
{
	  std::vector<MidgardBasicElement::st_zusatz> V=LernListen::getUeberlebenZusatz();
	  MBE_T::st_zusatz res=LSE_T::Auswaehlen(V, "In welcher Gegend beherrscht dieser Abenteurer 'Überleben' als Universelle Fertigkeit");
	  if (!res.mbe) return false;
	  MBEm::wie_erlernt wie;
	  wie.wie=MBEm::wie_erlernt::angeboren;
	  H_MBEm fert=LSE_T::lernen(res.mbe,wie,*this);
	  setUeberleben(fert);
	  return true;
}

// -1 uneindeutig
int Abenteurer::LandEindeutig() const
{
  if(Spezies()->Land()) return 1;
  if(Typ1()->Stadt()) return 0;
  if(Typ1()->Land()) return 1;
  return -1;
}
