// $Id: Abenteurer.hh,v 1.30 2004/12/22 08:10:30 christof Exp $
/*  Midgard Character Generator
 *  Copyright (C) 2002 Malte Thoma
 *  Copyright (C) 2003-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef CLASS_ABENTEURER
#define CLASS_ABENTEURER
#include <vector>
#include "Ausruestung.hh"
#include "Grundwerte.hh"
#include "Optionen.hh"
#include "Waffe.hh"
#include "Ruestung.hh"
#include "Enums.hh"
#include "Region.hh"
#include "ResistenzUndCo.hh"
#include <map>
#include <Misc/compiler_ports.h>
#include <time.h>
#include "AbenteurerLernpunkte.hh"

// Vielleicht sollte AbenteurerLernpunkte hier noch rein ...
class Abenteurer : public Grundwerte
{
   time_t last_saved_time;
   AusruestungBaum besitz;
   AbenteurerLernpunkte lernpunkte;

   std::list<H_MBEm> list_Beruf;
   std::list<H_MBEm> list_Fertigkeit_ang;
   std::list<H_MBEm> list_Fertigkeit;
   std::list<H_MBEm> list_WaffenGrund;
   std::list<H_MBEm> list_Waffen;
   std::list<H_WaffeBesitz> list_Waffen_besitz; // oder H_MBEm
   std::list<H_MBEm> list_Zauber;
   std::list<H_MBEm> list_Zauberwerk;
   std::list<H_MBEm> list_Kido;
   std::list<H_MBEm> list_Sprache;
   std::list<H_MBEm> list_Schrift;

   std::list<std::string> list_Gelernt_von_Spruchrolle;

public:
   typedef std::map<cH_Region,Model_copyable<bool> > regionen_t;

private:
   regionen_t regionen; // aktive Regionen
   Optionen optionen; // aktive Optionen

public:
   Abenteurer(bool initialize=false);

   const std::string LastSavedAt() const;
   void reset();
   void Originalregeln(); // alle Zusätze (Regionen etc.) abschalten
   bool Valid() const;

   __deprecated Grundwerte &getWerte() {return *this;}
   __deprecated const Grundwerte &getWerte() const {return *this;}
   AbenteurerLernpunkte const& getLernpunkte() const { return lernpunkte; }
   AbenteurerLernpunkte & getLernpunkte() { return lernpunkte; }

   AusruestungBaum &getBesitz(){return besitz;}
   AusruestungBaum& getAusruestung_as_parent(const std::string &name);
   AusruestungBaum* rekursiv(std::list<AusruestungBaum> &AB,const std::string &name);
   const AusruestungBaum &getBesitz() const {return besitz;}
   void setStandardAusruestung();
private:
   std::map<std::string,double> getBelastung() const;
public:
   double getBelastung(const std::string &s) const;
   int getNormallast() const;
   int getHoechstlast() const;
   int getSchublast() const;
   double getUeberlast() const;
   std::string Ruestung_B_Verlust(bool ueberlast_beruecksichtigen=false) const;


   void Steigertage2Alter();

   const std::string Beruf() const;
   const std::list<H_MBEm>& List_Beruf() const {return list_Beruf;}
   const std::list<H_MBEm>& List_Fertigkeit_ang() const {return list_Fertigkeit_ang;}
   const std::list<H_MBEm>& List_Fertigkeit() const {return list_Fertigkeit;}
   const std::list<H_MBEm>& List_WaffenGrund() const {return list_WaffenGrund;}
   const std::list<H_MBEm>& List_Waffen() const {return list_Waffen;}
   const std::list<H_WaffeBesitz>& List_Waffen_besitz() const {return list_Waffen_besitz;}
   const std::list<H_MBEm>& List_Zauber() const {return list_Zauber;}
   const std::list<H_MBEm>& List_Zauberwerk() const {return list_Zauberwerk;}
   const std::list<H_MBEm>& List_Kido() const {return list_Kido;}
   const std::list<H_MBEm>& List_Sprache() const {return list_Sprache;}
   const std::list<H_MBEm>& List_Schrift() const {return list_Schrift;}
   std::list<H_MBEm>& List_Beruf() {return list_Beruf;}
   std::list<H_MBEm>& List_Fertigkeit_ang() {return list_Fertigkeit_ang;}
   std::list<H_MBEm>& List_Fertigkeit()  {return list_Fertigkeit;}
   std::list<H_MBEm>& List_WaffenGrund() {return list_WaffenGrund;}
   std::list<H_MBEm>& List_Waffen()  {return list_Waffen;}
   std::list<H_WaffeBesitz>& List_Waffen_besitz() {return list_Waffen_besitz;}
   std::list<H_MBEm>& List_Zauber()  {return list_Zauber;}
   std::list<H_MBEm>& List_Zauberwerk()  {return list_Zauberwerk;}
   std::list<H_MBEm>& List_Kido()  {return list_Kido;}
   std::list<H_MBEm>& List_Sprache()  {return list_Sprache;}
   std::list<H_MBEm>& List_Schrift()  {return list_Schrift;}

   std::list<H_MBEm> &getList(MBE_T::MBEE was);
   std::list<H_MBEm> &get_known_list(const MBE_T::MBEE was);
   std::list<H_MBEm> const &get_known_list(const MBE_T::MBEE was) const;
   std::list<H_MBEm> &get_known_list(const H_MBEm &MBE);

   H_MBEm get_known(MBE_T::MBEE was, std::string const& name, std::string const& zusatz=std::string());
//   H_MBEm get_unknown(MBE_T::MBEE was, std::string const& name, std::string const& zusatz=std::string());
//   H_MBEm get_unknown(cH_MBE was, std::string const& zusatz=std::string()) const;
   bool lernbar(cH_MBE const& was) const;

   struct st_universell{H_MBEm mbe;bool voraussetzung;bool gelernt;
          st_universell(H_MBEm m)
            : mbe(m),voraussetzung(true),gelernt(false) {}};
   const std::list<st_universell> List_Universell() const;

private:
      class sort_universell {
        public: sort_universell() {}
        bool operator()(const st_universell &x,const st_universell &y) const;
      };
public:

   // alter Name
   __deprecated void speicherstream(std::ostream &datei) const;
   __deprecated bool xml_import_stream(std::istream &datei);
   // neuer Name
   void speichern(std::ostream &datei) const;
   bool laden(std::istream &datei);
private:
   void grundwerte_speichern(Tag &) const;
   void save_ausruestung(Tag &datei,const std::list<AusruestungBaum> &AB) const;

   void load_ausruestung(const Tag *tag, AusruestungBaum *AB);
   void load_fertigkeiten(const Tag *tag, const Tag *waffen_b, int xml_version);
   void load_regionen_optionen(const Tag *tag, int xml_version);

public:
   Optionen &getOptionen() { return optionen; }
   const Optionen &getOptionen() const { return optionen; }
   Model_ref<bool> getRegion(const cH_Region &r) { return regionen[r]; }
   const regionen_t &getRegionen() const {return regionen;}
   bool getRegion(const cH_Region &r) const;

   bool operator==(const Abenteurer& a) const
      {return Name_Abenteurer()==a.Name_Abenteurer() &&
              Version() == a.Version() ;}

   // (bool==true) => gelernt ; (bool==false) => universell(oder garnicht)
   const std::pair<int,bool> Erfolgswert(std::string name) const;
   const std::string SErfolgswert(std::string name) const;

   void setAngebFert();
   bool setAngebSinnFert(int wurf,const H_MBEm &MBE);
   void remove_WaffenGrund() ;

   static void move_element(std::list<H_MBEm>& von,
                            std::list<H_MBEm>& nach,
                            const H_MBEm& MBE);

   // neues Interface
   // true geklappt
   bool Steigern(H_MBEm &MBE);
   // alias für neu_lernen
   H_MBEm Erlernen(cH_MBE const& MBE, int bonus=0);
   // alles in einer Funktion
   H_MBEm Erlernen(H_LSE const& lse);
//   H_MBEm Erlernen(H_LSE const& lse, MBEm::wie_erlernt const& w, LSE_T::lernen_param const& param);
   // false: Fehlgeschlagen
   bool ReduzierenVerlernen(H_MBEm &MBE, bool &verlernt);

   void reduziere(H_MBEm &MBE);
   void verlerne(H_MBEm &MBE);
   bool steigere(H_MBEm &MBE);

   // Voraussetzungen prüfen und dann behandeln
   H_MBEm neu_lernen(cH_MBE const& MBE, int bonus=0);
   // (Kosten für) Ausdauer würfeln
   // ich empfehle H_MBEm m(ResistenzUndCo::Ausdauer); steigere(m);
   // CP: Dies scheint aber nicht zu würfeln ???
   __deprecated int ausdauer_steigern(int grad);
   // (Kosten für) Abwehr, Resistenz, Zaubern steigern
   // ich empfehle H_MBEm m(ResistenzUndCo::Zaubern); steigere(m);
   __deprecated int get_ab_re_za(const ResistenzUndCo::was_t was);
   __deprecated int get_ab_re_za(const H_MBEm &was);

   // komische public API ... CP
   __deprecated void move_neues_element(H_MBEm &MBE,std::list<H_MBEm> *MyList_neu);
private:
   __deprecated int get_ausdauer(int grad);  // heißt jetzt ausdauer_steigern

   void EP_aufwenden(MBE_T::EP_t ep_t,unsigned ep_k,MBEm::wie_erlernt::fp_t *fpptr);
   void PP_aufwenden(unsigned pp, const H_MBEm &MBE);

   // Kosten für reduzieren verrechnen?
   void desteigern(unsigned kosten);
   // Unterweisung, Selbststudium, Praxispunkte
   __deprecated bool steigern_usp(int &kosten,ResistenzUndCo::was_t was);
   bool steigern_usp(int &kosten,H_MBEm MBE,int &stufen,MBEm::wie_erlernt *erlernen=NULL);
   void add_lernzeit(int kosten,cH_MBE const& MBE);
   MBE_T::EP_t steigern_mit(const H_MBEm &MBE) const;
   bool genug_EP(int ep_k,MBE_T::EP_t mit) const;

   int genug_geld(int kosten) const;
   int EP_kosten(const int kosten) const;
   int PP_vorrat(const H_MBEm &MBE) const;

   // Beschreibung siehe .cc
   int stufen_auf_einmal_steigern_fuer_aep(const H_MBEm& MBE,int &kosten_out,int &aep) const;

public:
   void eigenschaften_steigern(int wurf=-1);
   bool eigenschaften_steigern_erlaubt() const;

   bool SpruchVonSpruchrolleGelernt(const std::string &zauber);
   bool ZauberSpruecheMitPP() const;

   unsigned VerfuegbareFP() const;

   // nutzen MBEm::Auswaehlen
   bool SpezialwaffeWaehlen();
   bool SpezialgebietWaehlen();
   bool HerkunftWaehlen();
   bool MutterspracheWaehlen();
   bool UeberlebenWaehlen();
   int LandEindeutig() const; // -1 uneindeutig, 0=Stadt
};

#endif
