// $Id: Abenteurer_steigern.cc,v 1.35 2006/08/24 17:14:21 thoma Exp $
/*  Midgard Character Generator
 *  Copyright (C) 2002 Malte Thoma
 *  Copyright (C) 2003-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Abenteurer.hh"
#include "Zauber.hh"
#include "Sprache.hh"
#include <Misc/itos.h>
#include "Random.hh"
#include "Datenbank.hh"
#include "Ausgabe.hh"
#include "magustrace.h"
#include <Misc/TraceNV.h>
#include <algorithm>

bool Abenteurer::ZauberSpruecheMitPP() const
{
   return Typ1()->SpruecheMitPP() || Typ2()->SpruecheMitPP() ;
}

// dies ist die eigentliche Steigern Funktion
// kosten sollte nach dieser Funktion die aufgewendeten GFP beinhalten (unnötig)
// Stufen enhält die Anzahl der Steigerungen
// usp = UnterweisungSelbststudiumPraxis
bool Abenteurer::steigern_usp(int &kosten,H_MBEm MBE,int &stufen,MBEm::wie_erlernt *erlernen)
{
  MBEm::wie_gesteigert wie;
  // EP
  MBE_T::EP_t ep_t=steigern_mit(MBE);
  int ep_k = 0; ep_kosten(kosten);
  MBEm::wie_erlernt::fp_t *fpptr=NULL;
  int oldfpanteil = fpanteil;
  int oldgoldanteil = goldanteil;
/*   if (wie_steigern==ws_Spruchrolle) fpptr=wie.spruchrolle_detail;
   else if (wie_steigern==ws_Praxis) fpptr=wie.praxis_detail.fp;
   else if (wie_steigern==ws_Unterweisung||wie_steigern==ws_Selbststudium)
     fpptr=wie.unterweisung_detail.fp;*/
  switch (wie_steigern)
  { case ws_Praxispunkte: ep_k=kosten;
      if (!erlernen)
      { wie.wie=MBEm::wie_erlernt::praxis;
        fpptr=&wie.praxis_detail.fp;
      }
      else
      { erlernen->wie=MBEm::wie_erlernt::praxis;
        fpptr=&erlernen->praxis_detail.fp;
      }
      break;
    case ws_Unterweisung:
      if (fpanteil!=0 && (fpanteil<33 || fpanteil>67))
      { Ausgabe(Ausgabe::Error,"Unterweisung: Ungültiger Anteil an FP "+itos(fpanteil));
        return false;
      }
      if (goldanteil>67)
      { Ausgabe(Ausgabe::Error,"Unterweisung: Ungültiger Anteil an Gold "+itos(goldanteil));
        return false;
      }
      ep_k=ep_kosten(kosten);
      if (!erlernen)
      { wie.wie=MBEm::wie_erlernt::unterweisung;
        fpptr=&wie.unterweisung_detail.fp;
      }
      else
      { erlernen->wie=MBEm::wie_erlernt::unterweisung;
        fpptr=&erlernen->unterweisung_detail.fp;
      }
      break;
    case ws_Selbststudium: goldanteil=0; fpanteil=100; ep_k=ep_kosten(kosten);
      if (!erlernen)
      { wie.wie=MBEm::wie_erlernt::selbststudium;
        fpptr=&wie.unterweisung_detail.fp;
      }
      else
      { erlernen->wie=MBEm::wie_erlernt::selbststudium;
        fpptr=&erlernen->unterweisung_detail.fp;
      }
      break;
    case ws_Spruchrolle: goldanteil=0; fpanteil=10; ep_k=(kosten+9)/10;
      assert(erlernen);
      erlernen->wie=MBEm::wie_erlernt::spruchrolle;
      break;
  }

  if(wie_steigern==ws_Praxispunkte)
  { int pp = PP_vorrat(MBE);
    if(pp==-1) return false;
    if(pp==0)
    { Ausgabe(Ausgabe::Error,(*MBE)->Name()+": Keine PP vorhanden");
      fpanteil=oldfpanteil;
      goldanteil=oldgoldanteil;
      return false;
    }
    if (wie_steigern==ws_Praxispunkte
          && wie_steigern_variante==wsv_NurPraxispunkte
          && pp*40<ep_k)
    { Ausgabe(Ausgabe::Error,(*MBE)->Name()+": Mindestens "+itos((ep_k+39)/40)
        + " Praxispunkte erforderlich (oder EP aufwenden)");
      fpanteil=oldfpanteil;
      goldanteil=oldgoldanteil;
      return false;
    }
    else if(ep_k > 2*pp*40)
    { Ausgabe(Ausgabe::Error,"Höchstens die Hälfte der GFP darf beim "
          "'Steigern mit PP' durch EP bestritten werden");
      Ausgabe(Ausgabe::Error,"Es fehlen "+itos((ep_k+79)/80 - pp)+"PP");
      fpanteil=oldfpanteil;
      goldanteil=oldgoldanteil;
      return false;
    }

    unsigned use_pp=0; // soviel PP werden verwendet

    if (MBE->What()==MBE_T::RESISTENZ_UND_CO
        || MBE->What()==MBE_T::ZAUBER)
    // immer genau EINE Stufe steigern, bzw einfach erlernen
    { stufen=1;
      use_pp = ep_k/40;
      ep_k  %= 40;
      // wenn PP+FP angewählt, aber kein PP aufgebraucht würde, trotzdem einen
      // nehmen, schließlich hat es der Benutzer so gesagt
      if (!use_pp || (wie_steigern_variante==wsv_NurPraxispunkte && ep_k))
      { ++use_pp;
        ep_k=0;
      }
    }
    else
    { ep_k = 40*pp;
      stufen=stufen_auf_einmal_steigern_fuer_aep(MBE,kosten,ep_k);
      use_pp=(kosten-ep_k+39)/40;
    }

    if (use_pp>unsigned(pp))
    { ep_k+=40*(use_pp-pp);
      use_pp=pp;
    }

    if(!genug_EP(ep_k,ep_t)) return false;
    ManuProC::Trace(LibMagus::trace_channel,__FUNCTION__,NV1(use_pp),NV1(ep_k),NV1(stufen));
    PP_aufwenden(use_pp,MBE);
    if (erlernen) erlernen->praxis_detail.praxispunkte=use_pp;
    else wie.praxis_detail.praxispunkte=use_pp;
  }
  else // Unterweisung, Selbststudium etc
  { if(!genug_EP(ep_k,ep_t)) return false;
    // genug GELD
    int gold_k=genug_geld(kosten);
    if (gold_k==-1) return false; // nicht genug Geld
    ManuProC::Trace(LibMagus::trace_channel,__FUNCTION__,NV1(gold_k),NV1(ep_k));
    addGold(-gold_k);
    if (erlernen) erlernen->unterweisung_detail.gs=gold_k;
    else wie.unterweisung_detail.gs=gold_k;
  }
  EP_aufwenden(ep_t,ep_k,fpptr);
  add_lernzeit(kosten,MBE->getMBE());
  if (wie_steigern==ws_Spruchrolle)
  { addGFP(ep_k*2);
//#warning list_Gelernt_von_Spruchrolle muss weg
    list_Gelernt_von_Spruchrolle.push_back(MBE->getMBE()->Name());
  }
  else addGFP(kosten);
  if (MBE->What()!=MBE_T::RESISTENZ_UND_CO
        && MBE->What()!=MBE_T::ZAUBER
        && !erlernen)
  {
    MBE->addErfolgswert(stufen,wie);
  }
  fpanteil=oldfpanteil;
  goldanteil=oldgoldanteil;
  return true;
}

void Abenteurer::EP_aufwenden(MBE_T::EP_t ep_t,unsigned ep_k,MBEm::wie_erlernt::fp_t *fpptr)
{ if(ep_t&MBE_T::KEP)
  { int kep=std::min<unsigned>(ep_k,KEP());
    addKEP(-kep);
    ep_k-=kep;
    if (fpptr) fpptr->kep=kep;
  }
  if(ep_t&MBE_T::ZEP)
  { int zep=std::min<unsigned>(ep_k,ZEP());
    addZEP(-zep);
    ep_k-=zep;
    if (fpptr) fpptr->zep=zep;
  }
  addAEP(-ep_k);
  if (fpptr) fpptr->aep=ep_k;
}

void Abenteurer::move_neues_element(H_MBEm &MBE,std::list<H_MBEm> *MyList_neu_)
{
 move_element(*MyList_neu_,getList((*MBE).What()),MBE);
}

void Abenteurer::add_lernzeit(int kosten, cH_MBE const& was)
{
  if(!!was && was->What()==MBE_T::RESISTENZ_UND_CO
     && was.getHandle<const ResistenzUndCo>()->WasIstEs()==ResistenzUndCo::eAusdauer)
  { addSteigertage(Grad_anstieg::AP_Maximum_Tage);
    return;
  }
  switch (wie_steigern)
  { case ws_Spruchrolle:
      addSteigertage(was.getHandle<const Zauber>()->iStufe()*3);
      break;
    case ws_Selbststudium:
      addSteigertage(kosten/5.);
      break;
    case ws_Unterweisung:
      addSteigertage(kosten/10.);
      break;
    case ws_Praxispunkte:
      addSteigertage(kosten/500.);
      break;
  }
}

//#warning das gehört in MBE!!!
MBE_T::EP_t Abenteurer::steigern_mit(const H_MBEm &MBE) const
{ switch (MBE->getMBE()->What())
  { case MBE_T::WAFFE:
    case MBE_T::WAFFEGRUND:
      return MBE_T::KEP;
    case MBE_T::ZAUBER:
    case MBE_T::ZAUBERWERK:
      return MBE_T::ZEP;
    case MBE_T::KIDO:
      return MBE_T::Beides;
    case MBE_T::RESISTENZ_UND_CO:
      switch (MBE->getHandle<ResistenzUndCo>()->WasIstEs())
      { case ResistenzUndCo::eAusdauer:
          return MBE_T::Beides;
        case ResistenzUndCo::eZaubern:
        case ResistenzUndCo::eResistenz:
          return MBE_T::ZEP;
        case ResistenzUndCo::eAbwehr:
          return MBE_T::KEP;
      }
      break;
    default:
      return MBE->getMBE()->Steigern_mit_EP();
  }
  abort();
}

// Rückgabewert: Kosten in GS oder -1 wenn es nicht reicht
int Abenteurer::genug_geld(const int kosten) const
{
  if (wie_steigern!=ws_Unterweisung) return 0; // keine Unterweisung => kein Geld nötig
  int gold_k = gold_kosten(kosten);
  if (!optionen.HausregelCheck(Optionen::Gold).active) gold_k*=10;

  std::pair<int,bool> gestu=Erfolgswert("Geschäftstüchtigkeit");
  if(gestu.second)
   {
      int iw=Random::W20();
      int erg=gestu.first+iw;
      Ausgabe(Ausgabe::Log,"EW:Geschäftstüchtigkeit = "+itos(gestu.first)+"+"+itos(iw)+"="+itos(erg));
      if(erg>=20)
       {
        int gold_gespart = int(gold_k*0.1);
        Ausgabe(Ausgabe::Log," => "+itos(gold_gespart)+" Gold gespart");
        gold_k-=gold_gespart;
       }
      else Ausgabe(Ausgabe::Log,"Das reicht nicht, kein Gold beim steigern gespart");
   }

  if (gold_k > Gold())
    { Ausgabe(Ausgabe::Error,"Zu wenig Gold um zu steigern, es fehlt "+itos(gold_k-Gold())+" Gold.");
      return -1;
    }
  else
   return gold_k;
}

bool Abenteurer::genug_EP(int ep_k,MBE_T::EP_t mit) const
{ int aep=AEP();
  std::string sw;
  if (mit&MBE_T::KEP) { aep += KEP(); sw =",KEP"; }
  if (mit&MBE_T::ZEP) { aep += ZEP(); sw +=",ZEP"; }
  if (ep_k > aep)
  { Ausgabe(Ausgabe::Error,"Zu wenig EP um zu steigern, es fehlen "+itos(ep_k-aep)+" Erfahrungspunkte (AEP"+sw+").");
    return false;
  }
  return true;
}

#if 0
int Abenteurer::EP_kosten(int kosten) const
{ if(wie_steigern==ws_Unterweisung || wie_steigern==ws_Selbststudium)
    return ep_kosten(kosten);
  else return kosten;
}
#endif

int Abenteurer::PP_vorrat(const H_MBEm &MBE) const
{
  if(wie_steigern!=ws_Praxispunkte)
    return 0;
  if(MBE->What()!=MBE_T::RESISTENZ_UND_CO)
  { if ((*MBE).What()!=MBE_T::ZAUBER) return MBE->Praxispunkte();
    return SpezialPP();
  }
  switch (MBE->getHandle<ResistenzUndCo>()->WasIstEs())
  { case ResistenzUndCo::eResistenz: return ResistenzPP() ;
    case ResistenzUndCo::eAbwehr: return AbwehrPP() ;
    case ResistenzUndCo::eZaubern: return ZaubernPP() ;
    case ResistenzUndCo::eAusdauer:
      Ausgabe(Ausgabe::Error,"Ausdauer kann nicht mit Praxispunkten gesteigert werden.");
      return -1;
    default: assert(!"Fehler in steigern_EP.cc");
  }
  abort();
}

void Abenteurer::PP_aufwenden(unsigned pp, const H_MBEm &MBE)
{
  if(MBE->What()!=MBE_T::RESISTENZ_UND_CO)
  { if ((*MBE).What()!=MBE_T::ZAUBER)
      MBE->setPraxispunkte(MBE->Praxispunkte()-pp);
    else addSpezialPP(-pp);
    return;
  }
  switch (MBE->getHandle<ResistenzUndCo>()->WasIstEs())
  { case ResistenzUndCo::eResistenz: addResistenzPP(-pp); break;
    case ResistenzUndCo::eAbwehr: addAbwehrPP(-pp); break;
    case ResistenzUndCo::eZaubern: addZaubernPP(-pp); break;
    default: assert(!"Fehler in PP_aufwenden");
  }
}

// kosten: out: Kosten an GFP
// aep: in: Verfügbare FP von PP
// aep: out: Benötigte EP
// result: steigerbare Stufen
int Abenteurer::stufen_auf_einmal_steigern_fuer_aep(const H_MBEm& _MBE,int &kosten,int &aep) const
{ ManuProC::Trace _t(LibMagus::trace_channel,__FUNCTION__,(*_MBE)->Name(),aep);
  kosten=0; // out parameter
  H_MBEm MBE=_MBE.dup();
  int stufen=0;
  int steiger_kosten = MBE->Steigern(*this);
  int ew=MBE->Erfolgswert();
  if(MBE->What()==MBE_T::FERTIGKEIT)
    ew=(*MBE)->FErfolgswert(*this,MBE);
  while (steiger_kosten<=aep && ew<(*MBE)->MaxErfolgswert(*this))
  { kosten+=steiger_kosten;
    ++stufen;
    aep-=steiger_kosten;

    MBE->addErfolgswert(1,MBEm::wie_gesteigert());
    ew=MBE->Erfolgswert();
    if (MBE->What()==MBE_T::FERTIGKEIT)
      ew=(*MBE)->FErfolgswert(*this,MBE);
    ManuProC::Trace(LibMagus::trace_channel,"",steiger_kosten,
               kosten,stufen,aep,ew,(*MBE)->MaxErfolgswert(*this));
    steiger_kosten = MBE->Steigern(*this);
  }
  if((steiger_kosten/2)<=aep && wie_steigern==ws_Praxispunkte
            && wie_steigern_variante==wsv_PraxispunkteFP
            && ew<(*MBE)->MaxErfolgswert(*this))
  { ManuProC::Trace(LibMagus::trace_channel,"<=½",steiger_kosten,aep);
    ++stufen;
    kosten+=steiger_kosten;
    aep=steiger_kosten-aep; // noch notwendige
  }
  else
  { ManuProC::Trace(LibMagus::trace_channel,">½",steiger_kosten,aep);
    aep=0;
  }
  ManuProC::Trace(LibMagus::trace_channel,"=",NV1(kosten),NV1(stufen),NV1(aep));
  return stufen;
}

// umbenannt von get_ausdauer
int Abenteurer::ausdauer_steigern(int grad)
{
   int bonus_K=0, bonus_aK=0, bonus_Z=0;
   int kosten = Datenbank.GradAnstieg.get_AP_Kosten(grad);
   switch (grad)
   { case  1: bonus_K =  4, bonus_aK =  3; bonus_Z =  2; break;
     default: if (2 <= grad && grad <= 10)
       { bonus_K = 3*grad, bonus_aK = 2*grad; bonus_Z = grad;  }
       else if (grad >=11)
       { bonus_K = 30, bonus_aK = 20; bonus_Z = 10;  }
       break;
   }
   if (!steigern_usp(kosten,ResistenzUndCo::eAusdauer)) return 0;
//   addGFP(kosten);
   int ap=0;
   for (int i=0;i<grad;++i) ap += Random::W6();

  int nab=0;
  if      (Typ1()->Ausdauer() == "k" || Typ2()->Ausdauer() == "k")  nab = bonus_K ;
  else if (Typ1()->Ausdauer() == "ak"|| Typ2()->Ausdauer() == "ak") nab = bonus_aK ;
  else  nab = bonus_Z ;
  int nap = ap + nab + bo_Au() ;
  int nspez = Grad()*Spezies()->AP_GradFak();
  nap += nspez;
  Ausgabe(Ausgabe::Error,"Ausdauerpunkte für Grad "+itos(Grad())+": "
   "Gewürfelt("+itos(ap)+") + Bonus für Typ("+itos(nab)
   +") + Persönlichen Bonus("+itos(bo_Au())
   +") + Spezies-Bonus("+itos(nspez)+") = "+itos(nap)+" AP");
   // Für alle ist die AP-anzahel mind. = Grad
  if (AP()<Grad()) setAP(Grad());
   // Neue AP höher als alte?
  if (nap>AP())  setAP(nap)  ;
  return kosten;
}

// was bedeutet denn das?
// get_Abwehr_Resistenz_Zaubern
// steigert und gibt Kosten zurück
int Abenteurer::get_ab_re_za(const ResistenzUndCo::was_t was)
{
  int alter_wert, max_wert;
  int kosten;
  int grad=Grad();
  // um die FP zu bestimmen, die angefallen waren, den Grad um 1 reduzieren
  if(reduzieren) --grad;

  if(grad==0) kosten=0;
  else switch (was)
  { case ResistenzUndCo::eAbwehr:
      max_wert = Datenbank.GradAnstieg.get_MaxAbwehr(grad);
      alter_wert = Abwehr_wert();
      kosten   = Datenbank.GradAnstieg.get_Abwehr_Kosten(alter_wert+1);
      if(reduzieren) kosten = Datenbank.GradAnstieg.get_Abwehr_Kosten(alter_wert);
      break;
    case ResistenzUndCo::eResistenz:
      max_wert = Datenbank.GradAnstieg.get_MaxResistenz(grad);
      alter_wert = Resistenz();
      kosten   = Datenbank.GradAnstieg.get_Resistenz_Kosten(alter_wert+1);
      if(reduzieren) kosten = Datenbank.GradAnstieg.get_Resistenz_Kosten(alter_wert);
      break;
    case ResistenzUndCo::eZaubern:
      if (is_mage())
       {
         max_wert = Datenbank.GradAnstieg.get_MaxZauber(grad);
         alter_wert = Zaubern_wert();
         kosten   = Datenbank.GradAnstieg.get_Zauber_Kosten(alter_wert+1);
         if(reduzieren) kosten = Datenbank.GradAnstieg.get_Zauber_Kosten(alter_wert);
       }
      else return 0;
      break;
    default: assert(!"never get here");
  }
  if(!kosten)
  { Ausgabe(Ausgabe::Error,"Minimaler Erfogswert erreicht");
    return 0;
  }

  if (alter_wert >= max_wert && !reduzieren)
  { Ausgabe(Ausgabe::Error,"Für Grad "+itos(Grad())+" ist der Maximalwert erreicht!") ;
    return 0;
  }

  else if(!reduzieren)
   { // hier stand eAusdauer ... was ist sinnvoller - oder?
     if (!steigern_usp(kosten,was)) return 0;
//     addGFP(kosten);
     if      (was==ResistenzUndCo::eAbwehr)    setAbwehr_wert(alter_wert+1);
     else if (was==ResistenzUndCo::eResistenz) setResistenz(alter_wert+1);
     else if (was==ResistenzUndCo::eZaubern)   setZaubern_wert(alter_wert+1);
   }
  else
   {
     addAEP(kosten); // streng genommen oft falsch (desteigern aber auch?)
     if      (was==ResistenzUndCo::eAbwehr)    setAbwehr_wert(alter_wert-1);
     else if (was==ResistenzUndCo::eResistenz) setResistenz(alter_wert-1);
     else if (was==ResistenzUndCo::eZaubern)   setZaubern_wert(alter_wert-1);
   }
  return kosten;
}

int Abenteurer::get_ab_re_za(const H_MBEm &MBE)
{ assert (MBE->What()==MBE_T::RESISTENZ_UND_CO);
  return get_ab_re_za(MBE->getHandle<ResistenzUndCo>()->WasIstEs());
}

bool Abenteurer::eigenschaften_steigern_erlaubt() const
{ return (Grad() > get_Grad_Basiswerte());
}

void Abenteurer::eigenschaften_steigern(int wurf)
{
  if(!eigenschaften_steigern_erlaubt())
   {Ausgabe(Ausgabe::Error,"Für Grad "+itos(get_Grad_Basiswerte())+" wurde schon gewürfelt");
    return;
   }
  // Erhöhen der Schicksalsgunst
  { int n=Datenbank.GradAnstieg.get_Schicksalsgunst(Grad());
    if(Spezies()->Name()=="Halbling") n+=2;
    add_SG(n);
  }

  if(wurf==-1) wurf=Random::W100();

  int z=wurf;
  std::string info="Beim Würfeln zur Erhöhung einer Eigenschaft für Grad "
      + itos(get_Grad_Basiswerte()+1) + " wurde eine ";
  info += itos(wurf);
  info +=" gewürfelt ==> ";
  std::string was = "keine Erhöhung";

  int erh = Random::W6()+1;
  int awko= Ko(); //alter_wert;
  int aapb = bo_Au(); // alter Wert
  if     ( z<=76 ) ;//nichts steigern
  else if( z<=78 ) { was="Stärke";           add_St(erh); }
  else if( z<=81 ) { was="Geschicklichkeit"; add_Gs(erh); }
  else if( z<=84 ) { was="Gewandheit"; add_Gw(erh); }
  else if( z<=87 ) { was="Konstitution"; add_Ko(erh); }
  else if( z<=90 ) { was="Intelligenz"; add_In(erh); }
  else if( z<=93 ) { was="Zaubertalent"; add_Zt(erh); }
  else if( z<=95 ) { was="Selbstbeherrschung"; add_Sb(erh); }
  else if( z<=97 ) { was="Willenskraft"; add_Wk(erh); }
  else if( z<=99 ) { was="persönliche Ausstrahlung"; add_pA(erh); }
  else if( z==100) { was="Aussehen"; add_Au(erh); }

  {
   //Setzen von abgeleiteten Werten, die durch eine Steigerung
   //bertoffen sein könnten:
   setSinn("Sechster Sinn",Zt()/25);
//   setRaufen((St()+Gw())/20+bo_An() );
   if(was=="Konstitution" && (awko/10 != Ko()/10))
         setLP(LP()-awko/10+Ko()/10);
   if( aapb!=bo_Au() )
      setAP(AP()-aapb+bo_Au());
  }

  info += was;
  if (was != "keine Erhöhung" )  info += " um "+itos(erh)+" erhöht.";
  Ausgabe(Ausgabe::Log,info);
  set_Grad_Basiswerte(1+get_Grad_Basiswerte());
}


void Abenteurer::Steigertage2Alter()
{
  float tage=Steigertage();
  int alter=Alter();
  int tage_pro_jahr=360;
  while(tage>tage_pro_jahr)
   {
     alter+=1;
     tage-=tage_pro_jahr;
   }
  setAlter(alter);
  setSteigertage(tage);
}

bool Abenteurer::steigere(H_MBEm &MBE)
{ if (MBE->What()==MBE_T::RESISTENZ_UND_CO)
  { cH_ResistenzUndCo r(MBE->getHandle<const ResistenzUndCo>());
    if (r->WasIstEs()==ResistenzUndCo::eAusdauer)
      return ausdauer_steigern(Grad())>0;
    else return get_ab_re_za(MBE)>0;
  }
  if ( MBE->Erfolgswert() >= (*MBE)->MaxErfolgswert(*this))
  { Ausgabe(Ausgabe::Error,(*MBE)->Name()+": Maximal möglicher Erfolgswert ("+itos((*MBE)->MaxErfolgswert(*this))+") erreicht");
    return false; }
  if(wie_steigern==ws_Unterweisung)
  { if( ((*MBE).What()==MBE_T::FERTIGKEIT &&
            MBE->Erfolgswert() >= cH_Fertigkeit(MBE->getMBE())->MaxUnterweisung()) ||
           ((*MBE).What()==MBE_T::SPRACHE &&
            MBE->Erfolgswert() >= cH_Fertigkeit("Sprache")->MaxUnterweisung()) ||
           ((*MBE).What()==MBE_T::SCHRIFT &&
            MBE->Erfolgswert() >= cH_Fertigkeit("Schreiben")->MaxUnterweisung())
          )
    { Ausgabe(Ausgabe::Error,(*MBE)->Name()+": Weitere Steigerung des Erfolgswertes ist NICHT mit Unterweisung möglich.");
      return false;
    }
  }
  int stufen=1;
  int steigerkosten=MBE->Steigern(*this);
  // Kopie anlegen, damit ältere Undo Schritte nicht erfasst werden
  if (MBE->What()!=MBE_T::RESISTENZ_UND_CO)
  { H_MBEm old=MBE;
    MBE=MBE.dup();
    std::list<H_MBEm> &L=getList(MBE->What());
    std::replace(L.begin(),L.end(),old,MBE);
  }
  if (!steigern_usp(steigerkosten,MBE,stufen)) return false;
//  addGFP(steigerkosten);
//  MBE->addErfolgswert(stufen);
  return true;
}

#if 0 // jetzt in Abenteurer::ReduzierenVerlernen
void Abenteurer::reduziere(H_MBEm &MBE)
{
  // Kopie anlegen, damit ältere Undo Schritte nicht erfasst werden
  if (MBE->What()!=MBE_T::RESISTENZ_UND_CO)
  { H_MBEm old=MBE;
    MBE=MBE.dup();
    std::list<H_MBEm> &L=getList(MBE->What());
    std::replace(L.begin(),L.end(),old,MBE);
  }
  if (!MBE->AlleSteigern().empty())
  {
    MBEm::wie_gesteigert ws=MBE->LetztesSteigern();
    MBE->AlleSteigern().erase(--MBE->AlleSteigern().end());
    MBE->setErfolgswert(ws.alter_wert);
    Ausgabe(Ausgabe::Warning, "Die Lerntage werden noch nicht wieder abgezogen");
    switch(ws.wie)
    {
      case MBEm::wie_erlernt::unterweisung:
        addAEP(ws.unterweisung_detail.fp.aep);
        addKEP(ws.unterweisung_detail.fp.kep);
        addZEP(ws.unterweisung_detail.fp.zep);
        addGold(ws.unterweisung_detail.gs);
        break;
      case MBEm::wie_erlernt::praxis:
        addAEP(ws.praxis_detail.fp.aep);
        addKEP(ws.praxis_detail.fp.kep);
        addZEP(ws.praxis_detail.fp.zep);
        MBE->addPraxispunkte(ws.praxis_detail.praxispunkte);
        break;
      default:
        Ausgabe(Ausgabe::Fatal, "unbekannte Lernmethode");
        break;
    }
  }
  else
  {
    Ausgabe(Ausgabe::Fatal, "Reduzieren ohne Informationen über das Lernen kann nicht richtig sein");
  }
}

// 2do: Aus Liste löschen?
void Abenteurer::verlerne(H_MBEm &MBE)
{ desteigern(MBE->Verlernen(*this));
}
#endif

// allgemeines Erlernen von Dingen, wird jetzt auch zur Liste hinzugefügt
H_MBEm Abenteurer::neu_lernen(cH_MBE const& MBE, int bonus)
{
 if(((MBE)->What()==MBE_T::FERTIGKEIT
   ||  (MBE)->What()==MBE_T::WAFFE)
      && !(MBE)->Voraussetzung(*this,false))
  {
    if((MBE)->Voraussetzung()=="schlank")
      Ausgabe(Ausgabe::Error,(MBE)->Name()+" können nur schlanke Abenteurer lernen");
    else Ausgabe(Ausgabe::Error,(MBE)->Name()+": Erst muss "+(MBE)->Voraussetzung()+" gelernt werden");
    return H_MBEm::null();
  }
 // Neue Dinge können nur durch Unterweisung gelernt werden
 // es sei denn es handelt sich um Zaubersprüche
 if((MBE)->What()==MBE_T::SPRACHE)
  { if (wie_steigern!=ws_Unterweisung && wie_steigern!=ws_Praxispunkte)
     { Ausgabe(Ausgabe::Error,(MBE)->Name()+": Neue Sprachen können nur durch Unterweisung oder Praxis gelernt werden");
       return H_MBEm::null();
     }
  }
 else if((MBE)->What()!=MBE_T::ZAUBER)
  { if (wie_steigern!=ws_Unterweisung)
     { Ausgabe(Ausgabe::Error,(MBE)->Name()+": Neue Fertigkeiten, Waffen und Schriften können nur durch 'Unterweisung' gelernt werden");
       return H_MBEm::null();
     }
  }
 else // ==> (*MBE).What()==MBE_T::ZAUBER
  {
    // Nicht alle Abenteurerklassen können Zauber auch mit Praxispunkten lernen
    if(wie_steigern==ws_Praxispunkte)
      { if(!Typ1()->SpruecheMitPP() && !Typ2()->SpruecheMitPP() )
           { Ausgabe(Ausgabe::Error,(MBE)->Name()+": Neue Zaubersprüche können von "+Typ1()->Name(Geschlecht())
                     +" nicht durch Praxispunkte gelernt werden");
             return H_MBEm::null();
           }
        if((MBE)->Standard__(*this)!="G")
           { Ausgabe(Ausgabe::Error,(MBE)->Name()+": Nur Grundzauber können von "+Typ1()->Name(Geschlecht())
                     +" mit Praxispunkten gelernt werden");
             return H_MBEm::null();
           }
      }
  }
 int kosten=(MBE)->Kosten(*this);

 // Lernen mit Spruchrolle: ///////////////////////////////////////////////
// if( (*MBE).What()==MBE_T::ZAUBER && wie_steigern==ws_Spruchrolle)
//      kosten/=10;
 /////////////////////////////////////////////////////////////////////////

 // nicht durch generelle Regel abgedeckt, da erst erlernt werden muss
 // Lernen neuer Sprache mit PP

 // wäre schöner, wenn das mit dem allgemeinen Code möglich wäre
 if((MBE)->What()==MBE_T::SPRACHE
       && wie_steigern==ws_Praxispunkte)
 { kosten=40;
   // 10 (+4) + 10 (+5) + 10 (+6) + 10 (+7) = 40FP
   // + 20 (+8) + 20 (+9) = 80FP/2 (bei Grundf.)
   H_MBEm MBEm(MBE);
   if((MBE)->Grundfertigkeit(*this)) MBEm->setErfolgswert(9);
   else MBEm->setErfolgswert(7);
   MBEm::wie_erlernt wie;
   wie.wie=MBEm::wie_erlernt::praxis;
   wie.praxis_detail.praxispunkte=1;
   MBEm->setLernArt(wie);
   add_lernzeit(kosten,MBE);
   addGFP(kosten);
   get_known_list(MBE->What()).push_back(MBEm);
   return MBEm;
 }
  if((MBE)->What()==MBE_T::ZAUBER
    && wie_steigern==ws_Spruchrolle
    && wie_steigern_variante!=wsv_SpruchrolleAlways
    && !MBE.getHandle<const Zauber>()->spruchrolle_wuerfeln(*this,bonus))
  { // fehlgeschlagener Versuch
    MBE_T::EP_t ep_t=steigern_mit(MBE);
    int ep_k = kosten/10;
    if (genug_EP(ep_k,ep_t))
    { EP_aufwenden(ep_t,ep_k,NULL);
      addGFP(ep_k);
      add_lernzeit(kosten,MBE);
    }
    return H_MBEm::null(); // Undo-Rollback ist hier eigentlich eine ganz schlechte Idee
  }

  H_MBEm MBEm(MBE);
  MBE_T::eZusatz was=MBE->ZusatzEnum(getVTyp());
  if(was)
  {
    MBE_T::st_zusatz zus=LernSchemaElement::Auswaehlen(MBE, *this);
    if (!zus.mbe && !zus.land && zus.name.empty()) return H_MBEm::null(); // keine Auswahl möglich?
    MBEm->setZusatz(zus);
  }
   MBEm::wie_erlernt wie;
   int dummy=1;
   if (!steigern_usp(kosten,MBEm,dummy,&wie)) return false;
   MBEm->setLernArt(wie);
   get_known_list((MBE)->What()).push_back(MBEm);
  // Lernen mit Spruchrolle: ///////////////////////////////////////////////
  return MBEm;
}

bool Abenteurer::steigern_usp(int &kosten, ResistenzUndCo::was_t was)
{ int d=1;
  H_MBEm mbe=H_MBEm(ResistenzUndCo::getMBE(was));
  // EW setzen?
  return steigern_usp(kosten,mbe,d);
}
