/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *  Copyright (C) 2003-2008 Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "KI.hh"
#include "Random.hh"
#include <Misc/itos.h>
#include "Zauber.hh"
#include "Fertigkeiten.hh"
#include <Ausgabe.hh>
//#include <iostream>
#include "LernListen.hh"
//#include "Prototyp2.hh"
//#include "Beruf.hh"
#include "Datenbank.hh"

//#define OLD

void MagusKI::VerteileGFP(int gfp,const Prozente100 &p,
                          const Grund_Standard_Ausnahme_MBE &gsa)
{
  prozente100=p;
  GSA_MBE=gsa;
  vec_Prototypen.clear();
  use_GSA_MBE=true;
  Verteile(gfp);
}

void MagusKI::VerteileGFP(int gfp,const Prozente100 &p,const std::vector<cH_Prototyp2> &Prototypen)
{
  prozente100=p;
  vec_Prototypen=Prototypen;
  use_GSA_MBE=false;
  Verteile(gfp);
}

#ifdef OLD
void MagusKI::ausgabe(const st_KI& stki, const bool gesteigert,MBE_T::MBEE was) const
{
  std::string aktion="neu gelernt";
  if(gesteigert) aktion="gesteigert";
  std::string prefix=was +" "+aktion ;
  if(stki.e_ki==OK)
   {
     Ausgabe(Ausgabe::Debug,prefix+"\tOK:\t"+stki.name+"\t"+itos(stki.wert));
   }
  else if(stki.e_ki==EmptyList)
   {
     Ausgabe(Ausgabe::Debug,prefix+ "\tEmptyList:");
   }
  else if(stki.e_ki==NotAllowedForGrad)
   {
     Ausgabe(Ausgabe::Debug,prefix+"\tNotAllowdForGrad:\t"+stki.name);
   }
  else if(stki.e_ki==Geheimzeichen)
   {
     Ausgabe(Ausgabe::Debug,prefix+"\tGeheimzeichen:\t"+stki.name);
   }
  else if(stki.e_ki==NoSteigern)
   {
     Ausgabe(Ausgabe::Debug,prefix+"\tNoSteigern:\t"+stki.name+"\t"+itos(stki.wert));
   }
  else assert(!"never get here\n");
}

#else // !OLD

struct lernbares_t {
  double accummulated_inverted_gfp; // denormalized probability
  cH_MBE neue_fertigkeit;
  H_MBEm steigernde_fertigkeit;

  lernbares_t() : accummulated_inverted_gfp(), neue_fertigkeit((MidgardBasicElement const*)0), steigernde_fertigkeit((MidgardBasicElement_mutable *)0) {}
  lernbares_t(double a, cH_MBE const& f) : accummulated_inverted_gfp(a), neue_fertigkeit(f), steigernde_fertigkeit((MidgardBasicElement_mutable *)0) {}
  lernbares_t(double a, H_MBEm const& f) : accummulated_inverted_gfp(a), neue_fertigkeit((MidgardBasicElement const*)0), steigernde_fertigkeit(f) {}
};

void erstelleListe(std::list<lernbares_t> &result, double &accumulator, std::list<cH_MBE> const& LL, Abenteurer const& Aben, unsigned gfp_limit, unsigned penalty=1)
{
  for (std::list<cH_MBE>::const_iterator i=LL.begin(); i!=LL.end(); ++i)
  {
    unsigned gfp= (*i)->Kosten(Aben)*penalty;
    if (gfp>gfp_limit) continue; // like in shorten_for_GFP
    if (!gfp) gfp=1;
    accumulator+=1.0/gfp;
    result.push_back(lernbares_t(accumulator,*i));
  }
}

void erstelleListe(std::list<lernbares_t> &result, double &accumulator, std::list<H_MBEm> const& LL, Abenteurer const& Aben, unsigned gfp_limit)
{
  for (std::list<H_MBEm>::const_iterator i=LL.begin(); i!=LL.end(); ++i)
  {
    unsigned gfp= (*i)->Steigern(Aben);
    if (gfp>gfp_limit) continue; // like in shorten_for_GFP
    if (!gfp) gfp=1;
    accumulator+=1.0/gfp;
    result.push_back(lernbares_t(accumulator,*i));
  }
}

std::list<lernbares_t> holeBewerteteGesamtliste(Abenteurer const& Aben, unsigned gfp_limit, double & accumulator)
{
  std::list<lernbares_t> result;
  accumulator=0;

  // neu erlernen …
  erstelleListe(result, accumulator, LernListen::get_steigern_MBE(Aben,MBE_T::FERTIGKEIT),Aben,gfp_limit);
  erstelleListe(result, accumulator, LernListen::get_steigern_MBE(Aben,MBE_T::WAFFE),Aben,gfp_limit);
  erstelleListe(result, accumulator, LernListen::get_steigern_MBE(Aben,MBE_T::SPRACHE),Aben,gfp_limit,100);
  erstelleListe(result, accumulator, LernListen::get_steigern_MBE(Aben,MBE_T::SCHRIFT),Aben,gfp_limit);
  erstelleListe(result, accumulator, LernListen::get_steigern_MBE(Aben,MBE_T::WAFFEGRUND),Aben,gfp_limit);
  if (Aben.is_mage())
  {
    bool salz=false;
    bool beschw=false;
    bool spruchrolle=false;
    erstelleListe(result, accumulator, LernListen::get_steigern_Zauberliste(Aben,salz,beschw,false,spruchrolle),Aben,gfp_limit);
    erstelleListe(result, accumulator, LernListen::get_steigern_ZauberWerkliste(Aben,false),Aben,gfp_limit);
  }

  // steigern
  erstelleListe(result, accumulator, Aben.get_known_list(MBE_T::FERTIGKEIT), Aben, gfp_limit);
  erstelleListe(result, accumulator, Aben.get_known_list(MBE_T::WAFFE), Aben, gfp_limit);
  erstelleListe(result, accumulator, Aben.get_known_list(MBE_T::SPRACHE), Aben, gfp_limit);
  erstelleListe(result, accumulator, Aben.get_known_list(MBE_T::SCHRIFT), Aben, gfp_limit);
//  erstelleListe(result, accumulator, Aben.get_known_list(MBE_T::KIDO), Aben, false, gfp_limit);
  return result;
}

#endif

void MagusKI::Verteile(int gfp)
{
  int count=0;//,gfpmem=gfp; // wenn man nicht mitzählt, kann es zu Endlosschleifen kommen
  const int MAXCOUNT=100;
  while(gfp>0 && count<MAXCOUNT)
   {
#ifndef OLD // new experimental method
    double maxval=0;
    int maxkosten = int(0.5*Datenbank.GradAnstieg.getGFP(Aben.Grad()+1));
    std::list<lernbares_t> lernbar= holeBewerteteGesamtliste(Aben, maxkosten, maxval);
    double randnum= rand()*(maxval/RAND_MAX);
    lernbares_t selected;
//printf("random %f out of %f\n", randnum, maxval);
    for (std::list<lernbares_t>::const_iterator i=lernbar.begin();i!=lernbar.end();++i)
    {
      if (randnum>=i->accummulated_inverted_gfp) selected=*i;
      else break;
    }
      if (!!selected.neue_fertigkeit)
      {
//printf("new %s\n", selected.neue_fertigkeit->Name().c_str());
//	if(selected.neue_fertigkeit->Name().substr(0,13)=="Geheimzeichen")  { ++count; continue; } // geht noch nicht
//	if (!selected.fertigkeit->VZusatz().empty())
//	{
//
//	}
	Aben.wie_steigern=Abenteurer::ws_Unterweisung;
	unsigned kosten=selected.neue_fertigkeit->Kosten(Aben);
	H_MBEm M=Aben.Erlernen(selected.neue_fertigkeit);
	if(!!M)
	  gfp-=kosten;
	else // ging nicht
	  ++count;
      }
      else if (!!selected.steigernde_fertigkeit)
      {
//	printf("inc %s\n", selected.steigernde_fertigkeit->getMBE()->Name().c_str());
        Aben.wie_steigern=Abenteurer::ws_Unterweisung;
        if (selected.steigernde_fertigkeit->getMBE()->What()==MBE_T::FERTIGKEIT
	      && selected.steigernde_fertigkeit->Erfolgswert() >= cH_Fertigkeit(selected.steigernde_fertigkeit->getMBE())->MaxUnterweisung())
	  Aben.wie_steigern= Abenteurer::ws_Selbststudium;
        unsigned kosten= selected.steigernde_fertigkeit->Steigern(Aben);
        bool ok=Aben.steigere(selected.steigernde_fertigkeit);
	if(ok)
	  gfp-=kosten;
	else // ging nicht
	  ++count;
      }
      else ++count;
#else
     int i=Random::W100();
     const MBE_T::MBEE was=Was();
     int spezial_allgemein=prozente100.getS(was);

     while(true)
      {
        st_KI stki(OK);
        if     (i<=spezial_allgemein) {stki=Steigern(gfp,was); ausgabe(stki,true,was); }
        else                          {stki=NeuLernen(gfp,was);ausgabe(stki,false,was); }
        if(stki.e_ki==OK) break;
        else if(stki.e_ki==EmptyList) break;
        else if(stki.e_ki==NotAllowedForGrad) ++count;
        else if(stki.e_ki==Geheimzeichen) ++count;
        else if(stki.e_ki==NoSteigern) ++count;
        else assert(!"never get here\n");
        if(count>=MAXCOUNT) break;
      }
#endif
     int kosten=teste_auf_gradanstieg();
     gfp-=kosten;
   }
  if(count==MAXCOUNT)
     Ausgabe(Ausgabe::Error,"Steigern abgebrochen, weil "+itos(count)+" Versuche erfolglos blieben.");
  // Das Alter anpassen
  Aben.Steigertage2Alter();
}

#ifdef OLD
static std::vector<H_MBEm> List_to_Vector(const std::list<H_MBEm> &L)
{
  std::vector<H_MBEm> V;
  for(std::list<H_MBEm>::const_iterator i=L.begin();i!=L.end();++i)
     V.push_back(*i);
  return V;
}

static std::vector<cH_MBE> List_to_Vector(const std::list<cH_MBE> &L)
{
  std::vector<cH_MBE> V;
  for(std::list<cH_MBE>::const_iterator i=L.begin();i!=L.end();++i)
     V.push_back(*i);
  return V;
}

const MBE_T::MBEE MagusKI::Was() const
{
  int z=Random::W100();
  const std::vector<Prozente100::st_was> V=prozente100.get100V();
  for(std::vector<Prozente100::st_was>::const_iterator i=V.begin();i!=V.end();++i)
    if( z <= i->prozent) return i->was;
  assert(!"never get here"); abort();
}

MagusKI::st_KI  MagusKI::NeuLernen(int &gfp,const MBE_T::MBEE was)
{
  std::list<cH_MBE> LL_=NeuLernenList(was,gfp);
  std::list<cH_MBE> LL;
  if(use_GSA_MBE)  LL=KI_GSA_Liste(LL_);
  else             LL=KI_Prototypen_Liste(was,LL_);
  if(LL.empty()) return st_KI(EmptyList);
  std::vector<cH_MBE> V=List_to_Vector(LL);
  int j=Random::integer(0,V.size()-1);
  cH_MBE M1=V[j];

  if((M1)->Name()=="Geheimzeichen")   return st_KI((M1)->Name(),Geheimzeichen);
  if(!allowed_for_grad(M1)) return st_KI((M1)->Name(),NotAllowedForGrad);

  Aben.wie_steigern=Abenteurer::ws_Unterweisung;
  H_MBEm M=Aben.Erlernen(M1);
  if(!!M)
   { gfp-=(M1)->Kosten(Aben);
     return st_KI((M1)->Name(),(*M).Erfolgswert(),OK);
   }
 return st_KI((M1)->Name(),NoSteigern); // ging nicht
}


MagusKI::st_KI MagusKI::Steigern(int &gfp,const MBE_T::MBEE was)
{
  std::list<H_MBEm> &LL_=Aben.get_known_list(was);
  std::list<H_MBEm> LL;
  if(use_GSA_MBE) LL=KI_GSA_Liste(LL_);
  else            LL=KI_Prototypen_Liste(was,LL_);
  if(LL.empty()) return st_KI(EmptyList);
  int j=Random::integer(0,LL.size()-1);
  int x=0;
  for(std::list<H_MBEm>::iterator i=LL.begin();i!=LL.end();++i)
   {
     if(j==x++)
      {
         if(!allowed_for_grad(*i)) return st_KI((*(*i))->Name(),NotAllowedForGrad);

         Aben.wie_steigern=Abenteurer::ws_Selbststudium;
         bool ok=Aben.steigere(*i);
         if(ok)
           { gfp-=(*i)->Steigern(Aben);
             return st_KI((*(*i))->Name(),(*(*i)).Erfolgswert(),OK);
           }
         else
             return st_KI((*(*i))->Name(),(*(*i)).Erfolgswert(),NoSteigern);
      }
   }
 assert(!"never get here");
 abort();
}


std::list<cH_MBE> MagusKI::NeuLernenList(const MBE_T::MBEE was,const int gfp) const
{
   std::list<cH_MBE> LL;
  switch (was) {
     case MBE_T::FERTIGKEIT:
     case MBE_T::WAFFE:
     case MBE_T::SPRACHE:
     case MBE_T::SCHRIFT:
     case MBE_T::WAFFEGRUND: LL=LernListen::get_steigern_MBE(Aben,was); break;
     case MBE_T::ZAUBER: {
         bool salz=false;
         bool beschw=false;
         bool spruchrolle=false;
         LL=LernListen::get_steigern_Zauberliste(Aben,salz,beschw,false,spruchrolle);
         break;
        }
     case MBE_T::ZAUBERWERK: LL=LernListen::get_steigern_ZauberWerkliste(Aben,false); break;
     default: assert(!"never get here\n");
   }
 LernListen::shorten_for_GFP(LL,Aben,gfp);
 return LL;
}


struct st_sort{std::string name; int kosten;
       st_sort(std::string n,int k) :name(n),kosten(k) {}
       bool operator<(const st_sort &b) const {return kosten<b.kosten ||
            (kosten==b.kosten && name<b.name); }
       };

// nie steigern
std::list<cH_MBE> MagusKI::KI_Prototypen_Liste(const MBE_T::MBEE was,const std::list<cH_MBE> &L)
{
  std::string Was;
  if     (was==MBE_T::FERTIGKEIT)Was="F";
  else if(was==MBE_T::ZAUBER)Was="Z";
  else return L;

  std::list<st_sort> S;
  for(std::list<cH_MBE>::const_iterator i=L.begin();i!=L.end();++i)
   {
     double fac=Prototyp2::fac_for(Was,((*i))->Name(),vec_Prototypen);
     int kosten = ((*i))->Kosten(Aben);
     S.push_back(st_sort(((*i))->Name(),kosten*fac));
   }
  S.sort();
  std::list<cH_MBE> newL;
  if(S.empty()) return newL;
  if     (was==MBE_T::FERTIGKEIT) newL.push_back(cH_Fertigkeit(S.begin()->name));
  else if(was==MBE_T::ZAUBER) newL.push_back(cH_Zauber(S.begin()->name));
  return newL;
}


std::list<cH_MBE> MagusKI::KI_GSA_Liste(const std::list<cH_MBE> &L)
{
  std::list<cH_MBE> Grund,Standard,Ausnahme;
  for(std::list<cH_MBE>::const_iterator i=L.begin();i!=L.end();++i)
   {
     if     (((*i))->Grundfertigkeit(Aben))    Grund.push_back(*i);
     else if(((*i))->Standardfertigkeit(Aben)) Standard.push_back(*i);
     else Ausnahme.push_back(*i);
   }
  int z=Random::W100();
  bool g=false,s=false,a=false;
  if     (z<GSA_MBE.getG()) g=true;
  else if(z<GSA_MBE.getG()+GSA_MBE.getS()) s=true;
  else a=true;

  if(g)
   { if     (!Grund.empty())    return Grund;
     else if(!Standard.empty()) return Standard;
     else                       return Ausnahme;
   }
  else if(s)
   { if     (!Standard.empty()) return Standard;
     else if(!Grund.empty())    return Grund;
     else                       return Ausnahme;
   }
  else if(a)
   {
     if     (!Ausnahme.empty()) return Ausnahme;
     else if(!Standard.empty()) return Standard;
     else                       return Grund;
   }
  assert(!"never get here\n");
  abort();
}

// immer steigern
std::list<H_MBEm> MagusKI::KI_Prototypen_Liste(const MBE_T::MBEE was,const std::list<H_MBEm> &L)
{
  std::string Was;
  if     (was==MBE_T::FERTIGKEIT)Was="F";
  else if(was==MBE_T::ZAUBER)Was="Z";
  else return L;

  std::list<st_sort> S;
  for(std::list<H_MBEm>::const_iterator i=L.begin();i!=L.end();++i)
   {
     double fac=Prototyp2::fac_for(Was,(*(*i))->Name(),vec_Prototypen);
     int kosten;
     kosten = (*i)->Steigern(Aben);
     S.push_back(st_sort((*(*i))->Name(),kosten*fac));
   }
  S.sort();
  std::list<H_MBEm> newL;
  if(S.empty()) return newL;
  if     (was==MBE_T::FERTIGKEIT) newL.push_back(H_MBEm(cH_Fertigkeit(S.begin()->name)));
  else if(was==MBE_T::ZAUBER) newL.push_back(H_MBEm(cH_Zauber(S.begin()->name)));
  return newL;
}


std::list<H_MBEm> MagusKI::KI_GSA_Liste(const std::list<H_MBEm> &L)
{
  std::list<H_MBEm> Grund,Standard,Ausnahme;
  for(std::list<H_MBEm>::const_iterator i=L.begin();i!=L.end();++i)
   {
     if     (((**i))->Grundfertigkeit(Aben))    Grund.push_back(*i);
     else if(((**i))->Standardfertigkeit(Aben)) Standard.push_back(*i);
     else Ausnahme.push_back(*i);
   }
  int z=Random::W100();
  bool g=false,s=false,a=false;
  if     (z<GSA_MBE.getG()) g=true;
  else if(z<GSA_MBE.getG()+GSA_MBE.getS()) s=true;
  else a=true;

  if(g)
   { if     (!Grund.empty())    return Grund;
     else if(!Standard.empty()) return Standard;
     else                       return Ausnahme;
   }
  else if(s)
   { if     (!Standard.empty()) return Standard;
     else if(!Grund.empty())    return Grund;
     else                       return Ausnahme;
   }
  else if(a)
   {
     if     (!Ausnahme.empty()) return Ausnahme;
     else if(!Standard.empty()) return Standard;
     else                       return Grund;
   }
  assert(!"never get here\n");
  abort();
}


bool MagusKI::allowed_for_grad(const H_MBEm &M)
{
  int maxkosten = int(0.5*Datenbank.GradAnstieg.getGFP(Aben.Grad()+1));
  if(M->Steigern(Aben)>maxkosten)
   {
Ausgabe(Ausgabe::Debug,(*M)->Name() +" wird nicht gesteigert, weil es "+itos((*M).Steigern(Aben))
+" kostet\tGrad: "+itos(Aben.Grad())+" "
+itos(Datenbank.GradAnstieg.getGFP(Aben.Grad()+1)));
     return false;
   }
  return true;
}

bool MagusKI::allowed_for_grad(const cH_MBE &M)
{
  int maxkosten = int(0.5*Datenbank.GradAnstieg.getGFP(Aben.Grad()+1));
  if((M)->Kosten(Aben)>maxkosten)
   {
Ausgabe(Ausgabe::Debug,(M)->Name() +" wird nicht neu gelernt, weil es "+itos((M)->Kosten(Aben))
+" kostet\tGrad: "+itos(Aben.Grad())+" "
+itos(Datenbank.GradAnstieg.getGFP(Aben.Grad()+1)));
     return false;
   }
  return true;
}

#endif

int MagusKI::teste_auf_gradanstieg()
{
  int oldgrad=Aben.Grad();
  Aben.setGrad(Datenbank.GradAnstieg.get_Grad(Aben.GFP()));
  int kosten=0;
  if(oldgrad!=Aben.Grad())
   { Aben.wie_steigern=Abenteurer::ws_Unterweisung;
     kosten+=Aben.ausdauer_steigern(Aben.Grad());
     kosten+=Aben.get_ab_re_za(ResistenzUndCo::eAbwehr);
     kosten+=Aben.get_ab_re_za(ResistenzUndCo::eResistenz);
     kosten+=Aben.get_ab_re_za(ResistenzUndCo::eZaubern);
     Aben.eigenschaften_steigern();
   }
  return kosten;
}

MagusKI::MagusKI(Abenteurer &a)
  : Aben(a), use_GSA_MBE(true)
{ a.fpanteil=50;
  a.goldanteil=50;
}
