/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _CLASS_SIMPLELERN_HH
#  define _CLASS_SIMPLELERN_HH

#include <RowDataBase.h>   
#include "Land.hh"
#include <Misc/EntryValueIntString.h>
#include "MidgardBasicElement.hh"
#include "Region.hh"
#include "Datenbank.hh"

////////////////////////////////////////////////////////////////
class Data_Zusatz : public RowDataBase
{
protected:
//      H_MBEm MBE;
      MBE_T::st_zusatz zusatz;
   public:
      Data_Zusatz(const MBE_T::st_zusatz &z)
         : zusatz(z){}

      enum Spalten {NAME,TYP,REGION};
      virtual cH_EntryValue Value(guint seqnr,gpointer gp) const
        {
         switch((Spalten)seqnr) 
           {
            case NAME : return cH_EntryValueIntString(zusatz.name);
            case TYP : return cH_EntryValueIntString(zusatz.typ);
            case REGION : return cH_EntryValueIntString(zusatz.long_region);
//            case REGION : return cH_EntryValueIntString(
//               Regionen_All::getRegionfromAbk(Database.Regionen,zusatz.region)->Name()+" "+zusatz.region_zusatz);
            default : return cH_EntryValue();
           }
          return cH_EntryValue();
        }
      const Handle<const MidgardBasicElement> &getMBE() const {return zusatz.mbe;}
      const MBE_T::st_zusatz &getZusatz() const {return zusatz;}
      bool Erlaubt() const {return zusatz.erlaubt;}
};

typedef Handle<const Data_Zusatz> cH_Data_Zusatz;
  
struct Data_Herkunft : Data_Zusatz
{
      Data_Herkunft(const MBE_T::st_zusatz &z)
         : Data_Zusatz(z){}

      enum Spalten {LAND,KONTINENT,SPRACHEN};
      virtual cH_EntryValue Value(guint seqnr,gpointer gp) const
        {
    	  cH_Land land = zusatz.land.cast_dynamic<Land const>();
         switch((Spalten)seqnr) 
           {
            case LAND      : return cH_EntryValueIntString(land->Name());
            case KONTINENT : return cH_EntryValueIntString(land->Kontinent());
            case SPRACHEN  : 
              { std::string s;
                std::vector<std::string> VS=land->Sprachen();
                for(std::vector<std::string>::const_iterator i=VS.begin();i!=VS.end();++i)
                  s+= *i + " ";
                return cH_EntryValueIntString(s);
              }
           }
          return cH_EntryValueIntString();
        }
};

typedef Handle<const Data_Herkunft> cH_Data_Herkunft;

#endif
