/*  Midgard Character Generator
 *  Copyright (C) 2001-2002 Malte Thoma
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"
#include "table_zufall.hh"
#include "midgard_CG.hh"
#include "Zufall.hh"
#include "KI.hh"
#include "Optionen.hh"
#include <Misc/itos.h>
#include "Prototyp.hh"
#include "Data_Prototyp.hh"
#include <libmagus/Ausgabe.hh>
#include <libmagus/zufall_steigern.hh>
#include <libmagus/Datenbank.hh>

void table_zufall::on_button_steigern_clicked()
{
  if(!hauptfenster->getAben().Typ1()->Valid())
   {
    Ausgabe(Ausgabe::Error,"Noch kein Abenteurer vorhanden.");
    return;
   }

  on_button_check100_clicked();
  int gfp;
  spinbutton_gfp->update();
  spinbutton_grad->update();
  if(radiobutton_steigern_gfp->get_active()) gfp=spinbutton_gfp->get_value_as_int();
  else gfp=zufall_steigern::GFPvonGrad(spinbutton_grad->get_value_as_int());

  zufall_steigern zs;
  zs.setzeProzente(prozente100);
  if(radiobutton_proto_verteilen->get_active())
     zs.setzeGSA(GSA_MBE);
  else
   {
     zs.setzePrototypen(getSelectedPrototypen());
   }
  zs.steigern(hauptfenster->getAben(),gfp);
}

std::vector<cH_Prototyp2> table_zufall::getSelectedPrototypen()
{
  std::vector<cH_Prototyp2> P;
  std::vector<cH_RowDataBase> VR=tree_prototyp->getSelectedRowDataBase_vec();
  for(std::vector<cH_RowDataBase>::const_iterator i=VR.begin();i!=VR.end();++i)
   {
     const Data_Prototyp *dt=dynamic_cast<const Data_Prototyp*>(&**i);     
     P.push_back(dt->getPrototyp());
   }
  return P;
}


void table_zufall::on_radiobutton_steigern_grad_toggled()
{
  if(radiobutton_steigern_grad->get_active())
   {
    spinbutton_grad->set_sensitive(true);
    spinbutton_grad->grab_focus();
    spinbutton_grad->select_region(0,-1);
    spinbutton_gfp->set_sensitive(false);
   }
}

void table_zufall::on_radiobutton_steigern_gfp_toggled()
{
  if(radiobutton_steigern_gfp->get_active())
   {
    spinbutton_gfp->set_sensitive(true);
    spinbutton_gfp->grab_focus();
    spinbutton_gfp->select_region(0,-1);
    spinbutton_grad->set_sensitive(false);
   }   
}


void table_zufall::fill_combo_steigern()
{
  static bool filled=false;
  if(filled) return;
  filled=true;
  std::list<std::string> L;
  const std::list<cH_Prototyp> &P=Datenbank.prototyp;
  for(std::list<cH_Prototyp>::const_iterator i=P.begin();i!=P.end();++i)
     L.push_back((*i)->Name());
  combo_prototyp->set_popdown_strings(L);
  if(!L.empty()) 
     combo_prototyp->get_entry()->set_text(*L.begin());
  on_button_check100_clicked();

 combo_prototyp->set_sensitive(true);
}

void table_zufall::on_combo_prototyp_activate()
{
}

bool table_zufall::on_combo_prototyp_focus_out_event(GdkEventFocus *ev)
{
  return 0;
}

bool table_zufall::entry_is_a_prototyp(const std::string &e)
{
  std::list<std::string> L;
  std::list<cH_Prototyp> P=Datenbank.prototyp;
  for(std::list<cH_Prototyp>::const_iterator i=P.begin();i!=P.end();++i)
     L.push_back((*i)->Name());
  if(find(L.begin(),L.end(),e)==L.end()) return false;
  return true;
}

void table_zufall::on_combo_prototyp_changed()
{
  std::string e=combo_prototyp->get_entry()->get_text();
  if(!entry_is_a_prototyp(e)) return;
  cH_Prototyp P(e);

  prozente100.set(MBE_T::FERTIGKEIT,P->Fertigkeit());
  prozente100.set(MBE_T::WAFFE,P->Waffe());
  prozente100.set(MBE_T::WAFFEGRUND,P->WaffeGrund());
  prozente100.set(MBE_T::ZAUBER,P->Zauber());
  prozente100.set(MBE_T::ZAUBERWERK,P->ZauberWerk());
  prozente100.set(MBE_T::SPRACHE,P->Sprache());
  prozente100.set(MBE_T::SCHRIFT,P->Schrift());
  prozente100.setS(MBE_T::FERTIGKEIT,P->FertSpezialist());
  prozente100.setS(MBE_T::WAFFE,P->WaffSpezialist());
  prozente100.setS(MBE_T::SPRACHE,P->SpraSpezialist());
  prozente100.setS(MBE_T::SCHRIFT,P->SchrSpezialist());

  prozente100.check100();
  Prototyp::setLast(prozente100);
   
  set_bereiche_spinbuttons();    
}

void table_zufall::set_bereiche_spinbuttons()
{
  spinbutton_fertigkeit->set_value(prozente100.get(MBE_T::FERTIGKEIT));
  spinbutton_waffen->set_value(prozente100.get(MBE_T::WAFFE));
  spinbutton_waffen_grund->set_value(prozente100.get(MBE_T::WAFFEGRUND));
  spinbutton_zauber->set_value(prozente100.get(MBE_T::ZAUBER));
  spinbutton_zauberwerk->set_value(prozente100.get(MBE_T::ZAUBERWERK));
  spinbutton_sprachen->set_value(prozente100.get(MBE_T::SPRACHE));
  spinbutton_schriften->set_value(prozente100.get(MBE_T::SCHRIFT));

  scale_fert->set_value(prozente100.getS(MBE_T::FERTIGKEIT));
  scale_waffen->set_value(prozente100.getS(MBE_T::WAFFE));
  scale_sprachen->set_value(prozente100.getS(MBE_T::SPRACHE));
  scale_schriften->set_value(prozente100.getS(MBE_T::SCHRIFT));

  spinbutton_grund->set_value(GSA_MBE.getG());
  spinbutton_standard->set_value(GSA_MBE.getS());
  spinbutton_ausnahme->set_value(GSA_MBE.getA());
}



void table_zufall::on_button_check100_clicked()
{
  spinbutton_fertigkeit->update();
  spinbutton_waffen->update();
  spinbutton_waffen_grund->update();
  spinbutton_zauber->update();
  spinbutton_zauberwerk->update();
  spinbutton_sprachen->update();
  spinbutton_schriften->update();
  prozente100.set(MBE_T::FERTIGKEIT,spinbutton_fertigkeit->get_value_as_int());
  prozente100.set(MBE_T::WAFFE,spinbutton_waffen->get_value_as_int());
  prozente100.set(MBE_T::WAFFEGRUND,spinbutton_waffen_grund->get_value_as_int());
  prozente100.set(MBE_T::ZAUBER,spinbutton_zauber->get_value_as_int());
  prozente100.set(MBE_T::ZAUBERWERK,spinbutton_zauberwerk->get_value_as_int());
  prozente100.set(MBE_T::SPRACHE,spinbutton_sprachen->get_value_as_int());
  prozente100.set(MBE_T::SCHRIFT,spinbutton_schriften->get_value_as_int());
  prozente100.check100();
  
  spinbutton_grund->update();
  spinbutton_standard->update();
  spinbutton_ausnahme->update();
  GSA_MBE.setG(spinbutton_grund->get_value_as_int());
  GSA_MBE.setS(spinbutton_standard->get_value_as_int());
  GSA_MBE.setA(spinbutton_ausnahme->get_value_as_int());

  set_bereiche_spinbuttons();    
  Prototyp::setLast(prozente100);

}


void table_zufall::on_scale_fert_activate()
{
  prozente100.setS(MBE_T::FERTIGKEIT,scale_fert->get_value());
}
void table_zufall::on_scale_waffen_activate()
{
  prozente100.setS(MBE_T::WAFFE,scale_waffen->get_value());
}
void table_zufall::on_scale_schriften_activate()
{
  prozente100.setS(MBE_T::SCHRIFT,scale_schriften->get_value());
}
void table_zufall::on_scale_sprachen_activate()
{
  prozente100.setS(MBE_T::SPRACHE,scale_sprachen->get_value());
}

void table_zufall::on_togglebutton_prototyp_toggled()
{
  if(togglebutton_prototyp->get_active()) frame_prototyp_mod->show();
  else frame_prototyp_mod->hide();
}

void table_zufall::on_spinbutton_fertigkeit_activate()
{  
  spinbutton_waffen->grab_focus();
  spinbutton_waffen->select_region(0,-1);
}

void table_zufall::on_spinbutton_waffen_activate()
{  
  spinbutton_waffen_grund->grab_focus();
  spinbutton_waffen_grund->select_region(0,-1);
}

void table_zufall::on_spinbutton_waffen_grund_activate()
{  
  spinbutton_zauber->grab_focus();
  spinbutton_zauber->select_region(0,-1);
}

void table_zufall::on_spinbutton_zauber_activate()
{  
  spinbutton_zauberwerk->grab_focus();
  spinbutton_zauberwerk->select_region(0,-1);
}

void table_zufall::on_spinbutton_zauberwerk_activate()
{  
  spinbutton_sprachen->grab_focus();
  spinbutton_sprachen->select_region(0,-1);
}

void table_zufall::on_spinbutton_sprachen_activate()
{  
  spinbutton_schriften->grab_focus();
  spinbutton_schriften->select_region(0,-1);
}

void table_zufall::on_spinbutton_schriften_activate()
{  
 on_button_check100_clicked();
}

void table_zufall::on_spinbutton_grund_activate()
{
 spinbutton_standard->grab_focus();
 spinbutton_standard->select_region(0,-1);
}

void table_zufall::on_spinbutton_standard_activate()
{
 spinbutton_ausnahme->grab_focus();
 spinbutton_ausnahme->select_region(0,-1);
}

void table_zufall::on_spinbutton_ausnahme_activate()
{
  on_button_check100_clicked();
}

